import jinja2
import pathlib
from .storage import async_setup_frontend_storage as async_setup_frontend_storage
from aiohttp import web, web_urldispatcher
from collections.abc import Iterator
from homeassistant.components import websocket_api as websocket_api
from homeassistant.components.http.view import HomeAssistantView as HomeAssistantView
from homeassistant.components.websocket_api.connection import ActiveConnection as ActiveConnection
from homeassistant.config import async_hass_config_yaml as async_hass_config_yaml
from homeassistant.const import CONF_MODE as CONF_MODE, CONF_NAME as CONF_NAME, EVENT_THEMES_UPDATED as EVENT_THEMES_UPDATED
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall, callback as callback
from homeassistant.helpers import service as service
from homeassistant.helpers.translation import async_get_translations as async_get_translations
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.loader import async_get_integration as async_get_integration, bind_hass as bind_hass
from typing import Any, TypedDict
from yarl import URL

DOMAIN: str
CONF_THEMES: str
CONF_THEMES_MODES: str
CONF_THEMES_LIGHT: str
CONF_THEMES_DARK: str
CONF_EXTRA_HTML_URL: str
CONF_EXTRA_HTML_URL_ES5: str
CONF_EXTRA_MODULE_URL: str
CONF_EXTRA_JS_URL_ES5: str
CONF_FRONTEND_REPO: str
CONF_JS_VERSION: str
EVENT_PANELS_UPDATED: str
DEFAULT_THEME_COLOR: str
DATA_PANELS: str
DATA_JS_VERSION: str
DATA_EXTRA_MODULE_URL: str
DATA_EXTRA_JS_URL_ES5: str
THEMES_STORAGE_KEY: Any
THEMES_STORAGE_VERSION: int
THEMES_SAVE_DELAY: int
DATA_THEMES_STORE: str
DATA_THEMES: str
DATA_DEFAULT_THEME: str
DATA_DEFAULT_DARK_THEME: str
DEFAULT_THEME: str
VALUE_NO_THEME: str
PRIMARY_COLOR: str
_LOGGER: Any
EXTENDED_THEME_SCHEMA: Any
THEME_SCHEMA: Any
CONFIG_SCHEMA: Any
SERVICE_SET_THEME: str
SERVICE_RELOAD_THEMES: str

class Manifest:
    manifest: Any
    def __init__(self, data: dict) -> None: ...
    def __getitem__(self, key: str) -> Any: ...
    @property
    def json(self) -> str: ...
    _serialized: Any
    def _serialize(self) -> None: ...
    def update_key(self, key: str, val: str) -> None: ...

MANIFEST_JSON: Any

class UrlManager:
    urls: Any
    def __init__(self, urls: list[str]) -> None: ...
    def add(self, url: str) -> None: ...
    def remove(self, url: str) -> None: ...

class Panel:
    component_name: str
    sidebar_icon: Union[str, None]
    sidebar_title: Union[str, None]
    frontend_url_path: Union[str, None]
    config: Union[dict[str, Any], None]
    require_admin: bool
    def __init__(self, component_name: str, sidebar_title: Union[str, None], sidebar_icon: Union[str, None], frontend_url_path: Union[str, None], config: Union[dict[str, Any], None], require_admin: bool) -> None: ...
    def to_response(self) -> PanelRespons: ...

def async_register_built_in_panel(hass: HomeAssistant, component_name: str, sidebar_title: Union[str, None] = ..., sidebar_icon: Union[str, None] = ..., frontend_url_path: Union[str, None] = ..., config: Union[dict[str, Any], None] = ..., require_admin: bool = ..., *, update: bool = ...) -> None: ...
def async_remove_panel(hass: HomeAssistant, frontend_url_path: str) -> None: ...
def add_extra_js_url(hass: HomeAssistant, url: str, es5: bool = ...) -> None: ...
def add_manifest_json_key(key: str, val: Any) -> None: ...
def _frontend_root(dev_repo_path: Union[str, None]) -> pathlib.Path: ...
async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def _async_setup_themes(hass: HomeAssistant, themes: Union[dict[str, Any], None]) -> None: ...
def _async_render_index_cached(template: jinja2.Template, **kwargs: Any) -> str: ...

class IndexView(web_urldispatcher.AbstractResource):
    repo_path: Any
    hass: Any
    _template_cache: Any
    def __init__(self, repo_path: Union[str, None], hass: HomeAssistant) -> None: ...
    @property
    def canonical(self) -> str: ...
    @property
    def _route(self) -> web_urldispatcher.ResourceRoute: ...
    def url_for(self, **kwargs: str) -> URL: ...
    async def resolve(self, request: web.Request) -> tuple[Union[web_urldispatcher.UrlMappingMatchInfo, None], set[str]]: ...
    def add_prefix(self, prefix: str) -> None: ...
    def get_info(self) -> dict[str, list[str]]: ...
    def freeze(self) -> None: ...
    def raw_match(self, path: str) -> bool: ...
    def get_template(self) -> jinja2.Template: ...
    async def get(self, request: web.Request) -> web.Response: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[web_urldispatcher.ResourceRoute]: ...

class ManifestJSONView(HomeAssistantView):
    requires_auth: bool
    url: str
    name: str
    def get(self, request: web.Request) -> web.Response: ...

def websocket_get_panels(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
def websocket_get_themes(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
async def websocket_get_translations(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
async def websocket_get_version(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...

class PanelRespons(TypedDict):
    component_name: str
    icon: Union[str, None]
    title: Union[str, None]
    config: Union[dict[str, Any], None]
    url_path: Union[str, None]
    require_admin: bool



from homeassistant.const import STATE_OFF as STATE_OFF, STATE_ON as STATE_ON
from typing import Final

CONF_WRITE_TIMEOUT: Final[str]
DEFAULT_NAME: Final[str]
DEFAULT_TIMEOUT: Final[int]
DEFAULT_WRITE_TIMEOUT: Final[int]
ECO_MODE: Final[str]
ICON: Final[str]
INPUT_SOURCE: Final[str]
LAMP: Final[str]
LAMP_HOURS: Final[str]
MODEL: Final[str]
CMD_DICT: Final[dict[str, str]]



import datetime as dt
from .const import ATTR_APP_ID as ATTR_APP_ID, ATTR_APP_NAME as ATTR_APP_NAME, ATTR_GROUP_MEMBERS as ATTR_GROUP_MEMBERS, ATTR_INPUT_SOURCE as ATTR_INPUT_SOURCE, ATTR_INPUT_SOURCE_LIST as ATTR_INPUT_SOURCE_LIST, ATTR_MEDIA_ALBUM_ARTIST as ATTR_MEDIA_ALBUM_ARTIST, ATTR_MEDIA_ALBUM_NAME as ATTR_MEDIA_ALBUM_NAME, ATTR_MEDIA_ARTIST as ATTR_MEDIA_ARTIST, ATTR_MEDIA_CHANNEL as ATTR_MEDIA_CHANNEL, ATTR_MEDIA_CONTENT_ID as ATTR_MEDIA_CONTENT_ID, ATTR_MEDIA_CONTENT_TYPE as ATTR_MEDIA_CONTENT_TYPE, ATTR_MEDIA_DURATION as ATTR_MEDIA_DURATION, ATTR_MEDIA_ENQUEUE as ATTR_MEDIA_ENQUEUE, ATTR_MEDIA_EPISODE as ATTR_MEDIA_EPISODE, ATTR_MEDIA_EXTRA as ATTR_MEDIA_EXTRA, ATTR_MEDIA_PLAYLIST as ATTR_MEDIA_PLAYLIST, ATTR_MEDIA_POSITION as ATTR_MEDIA_POSITION, ATTR_MEDIA_POSITION_UPDATED_AT as ATTR_MEDIA_POSITION_UPDATED_AT, ATTR_MEDIA_REPEAT as ATTR_MEDIA_REPEAT, ATTR_MEDIA_SEASON as ATTR_MEDIA_SEASON, ATTR_MEDIA_SEEK_POSITION as ATTR_MEDIA_SEEK_POSITION, ATTR_MEDIA_SERIES_TITLE as ATTR_MEDIA_SERIES_TITLE, ATTR_MEDIA_SHUFFLE as ATTR_MEDIA_SHUFFLE, ATTR_MEDIA_TITLE as ATTR_MEDIA_TITLE, ATTR_MEDIA_TRACK as ATTR_MEDIA_TRACK, ATTR_MEDIA_VOLUME_LEVEL as ATTR_MEDIA_VOLUME_LEVEL, ATTR_MEDIA_VOLUME_MUTED as ATTR_MEDIA_VOLUME_MUTED, ATTR_SOUND_MODE as ATTR_SOUND_MODE, ATTR_SOUND_MODE_LIST as ATTR_SOUND_MODE_LIST, DOMAIN as DOMAIN, MEDIA_CLASS_DIRECTORY as MEDIA_CLASS_DIRECTORY, REPEAT_MODES as REPEAT_MODES, SERVICE_CLEAR_PLAYLIST as SERVICE_CLEAR_PLAYLIST, SERVICE_JOIN as SERVICE_JOIN, SERVICE_PLAY_MEDIA as SERVICE_PLAY_MEDIA, SERVICE_SELECT_SOUND_MODE as SERVICE_SELECT_SOUND_MODE, SERVICE_SELECT_SOURCE as SERVICE_SELECT_SOURCE, SERVICE_UNJOIN as SERVICE_UNJOIN, SUPPORT_BROWSE_MEDIA as SUPPORT_BROWSE_MEDIA, SUPPORT_CLEAR_PLAYLIST as SUPPORT_CLEAR_PLAYLIST, SUPPORT_GROUPING as SUPPORT_GROUPING, SUPPORT_NEXT_TRACK as SUPPORT_NEXT_TRACK, SUPPORT_PAUSE as SUPPORT_PAUSE, SUPPORT_PLAY as SUPPORT_PLAY, SUPPORT_PLAY_MEDIA as SUPPORT_PLAY_MEDIA, SUPPORT_PREVIOUS_TRACK as SUPPORT_PREVIOUS_TRACK, SUPPORT_REPEAT_SET as SUPPORT_REPEAT_SET, SUPPORT_SEEK as SUPPORT_SEEK, SUPPORT_SELECT_SOUND_MODE as SUPPORT_SELECT_SOUND_MODE, SUPPORT_SELECT_SOURCE as SUPPORT_SELECT_SOURCE, SUPPORT_SHUFFLE_SET as SUPPORT_SHUFFLE_SET, SUPPORT_STOP as SUPPORT_STOP, SUPPORT_TURN_OFF as SUPPORT_TURN_OFF, SUPPORT_TURN_ON as SUPPORT_TURN_ON, SUPPORT_VOLUME_MUTE as SUPPORT_VOLUME_MUTE, SUPPORT_VOLUME_SET as SUPPORT_VOLUME_SET, SUPPORT_VOLUME_STEP as SUPPORT_VOLUME_STEP
from .errors import BrowseError as BrowseError
from aiohttp import web
from aiohttp.typedefs import LooseHeaders as LooseHeaders
from homeassistant.components import websocket_api as websocket_api
from homeassistant.components.http import HomeAssistantView as HomeAssistantView, KEY_AUTHENTICATED as KEY_AUTHENTICATED
from homeassistant.components.websocket_api.const import ERR_NOT_FOUND as ERR_NOT_FOUND, ERR_NOT_SUPPORTED as ERR_NOT_SUPPORTED, ERR_UNKNOWN_ERROR as ERR_UNKNOWN_ERROR
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import HTTP_INTERNAL_SERVER_ERROR as HTTP_INTERNAL_SERVER_ERROR, HTTP_NOT_FOUND as HTTP_NOT_FOUND, HTTP_OK as HTTP_OK, HTTP_UNAUTHORIZED as HTTP_UNAUTHORIZED, SERVICE_MEDIA_NEXT_TRACK as SERVICE_MEDIA_NEXT_TRACK, SERVICE_MEDIA_PAUSE as SERVICE_MEDIA_PAUSE, SERVICE_MEDIA_PLAY as SERVICE_MEDIA_PLAY, SERVICE_MEDIA_PLAY_PAUSE as SERVICE_MEDIA_PLAY_PAUSE, SERVICE_MEDIA_PREVIOUS_TRACK as SERVICE_MEDIA_PREVIOUS_TRACK, SERVICE_MEDIA_SEEK as SERVICE_MEDIA_SEEK, SERVICE_MEDIA_STOP as SERVICE_MEDIA_STOP, SERVICE_REPEAT_SET as SERVICE_REPEAT_SET, SERVICE_SHUFFLE_SET as SERVICE_SHUFFLE_SET, SERVICE_TOGGLE as SERVICE_TOGGLE, SERVICE_TURN_OFF as SERVICE_TURN_OFF, SERVICE_TURN_ON as SERVICE_TURN_ON, SERVICE_VOLUME_DOWN as SERVICE_VOLUME_DOWN, SERVICE_VOLUME_MUTE as SERVICE_VOLUME_MUTE, SERVICE_VOLUME_SET as SERVICE_VOLUME_SET, SERVICE_VOLUME_UP as SERVICE_VOLUME_UP, STATE_IDLE as STATE_IDLE, STATE_OFF as STATE_OFF, STATE_PLAYING as STATE_PLAYING
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE, datetime as datetime
from homeassistant.helpers.entity import Entity as Entity, EntityDescription as EntityDescription
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.network import get_url as get_url
from homeassistant.loader import bind_hass as bind_hass
from typing import Any

_LOGGER: Any
ENTITY_ID_FORMAT: Any
CACHE_IMAGES: str
CACHE_MAXSIZE: str
CACHE_LOCK: str
CACHE_URL: str
CACHE_CONTENT: str
ENTITY_IMAGE_CACHE: Any
SCAN_INTERVAL: Any
DEVICE_CLASS_TV: str
DEVICE_CLASS_SPEAKER: str
DEVICE_CLASS_RECEIVER: str
DEVICE_CLASSES: Any
DEVICE_CLASSES_SCHEMA: Any
MEDIA_PLAYER_PLAY_MEDIA_SCHEMA: Any
ATTR_TO_PROPERTY: Any

def is_on(hass, entity_id: Any | None = ...): ...
def _rename_keys(**keys): ...
async def async_setup(hass, config): ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class MediaPlayerEntityDescription(EntityDescription): ...

class MediaPlayerEntity(Entity):
    entity_description: MediaPlayerEntityDescription
    _access_token: Union[str, None]
    _attr_app_id: Union[str, None]
    _attr_app_name: Union[str, None]
    _attr_group_members: Union[list[str], None]
    _attr_is_volume_muted: Union[bool, None]
    _attr_media_album_artist: Union[str, None]
    _attr_media_album_name: Union[str, None]
    _attr_media_artist: Union[str, None]
    _attr_media_channel: Union[str, None]
    _attr_media_content_id: Union[str, None]
    _attr_media_content_type: Union[str, None]
    _attr_media_duration: Union[int, None]
    _attr_media_episode: Union[str, None]
    _attr_media_image_hash: Union[str, None]
    _attr_media_image_remotely_accessible: bool
    _attr_media_image_url: Union[str, None]
    _attr_media_playlist: Union[str, None]
    _attr_media_position_updated_at: Union[dt.datetime, None]
    _attr_media_position: Union[int, None]
    _attr_media_season: Union[str, None]
    _attr_media_series_title: Union[str, None]
    _attr_media_title: Union[str, None]
    _attr_media_track: Union[int, None]
    _attr_repeat: Union[str, None]
    _attr_shuffle: Union[bool, None]
    _attr_sound_mode_list: Union[list[str], None]
    _attr_sound_mode: Union[str, None]
    _attr_source_list: Union[list[str], None]
    _attr_source: Union[str, None]
    _attr_state: Union[str, None]
    _attr_supported_features: int
    _attr_volume_level: Union[float, None]
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def access_token(self) -> str: ...
    @property
    def volume_level(self) -> Union[float, None]: ...
    @property
    def is_volume_muted(self) -> Union[bool, None]: ...
    @property
    def media_content_id(self) -> Union[str, None]: ...
    @property
    def media_content_type(self) -> Union[str, None]: ...
    @property
    def media_duration(self) -> Union[int, None]: ...
    @property
    def media_position(self) -> Union[int, None]: ...
    @property
    def media_position_updated_at(self) -> Union[dt.datetime, None]: ...
    @property
    def media_image_url(self) -> Union[str, None]: ...
    @property
    def media_image_remotely_accessible(self) -> bool: ...
    @property
    def media_image_hash(self) -> Union[str, None]: ...
    async def async_get_media_image(self): ...
    async def async_get_browse_image(self, media_content_type: str, media_content_id: str, media_image_id: Union[str, None] = ...) -> tuple[Union[str, None], Union[str, None]]: ...
    @property
    def media_title(self) -> Union[str, None]: ...
    @property
    def media_artist(self) -> Union[str, None]: ...
    @property
    def media_album_name(self) -> Union[str, None]: ...
    @property
    def media_album_artist(self) -> Union[str, None]: ...
    @property
    def media_track(self) -> Union[int, None]: ...
    @property
    def media_series_title(self) -> Union[str, None]: ...
    @property
    def media_season(self) -> Union[str, None]: ...
    @property
    def media_episode(self) -> Union[str, None]: ...
    @property
    def media_channel(self) -> Union[str, None]: ...
    @property
    def media_playlist(self) -> Union[str, None]: ...
    @property
    def app_id(self) -> Union[str, None]: ...
    @property
    def app_name(self) -> Union[str, None]: ...
    @property
    def source(self) -> Union[str, None]: ...
    @property
    def source_list(self) -> Union[list[str], None]: ...
    @property
    def sound_mode(self) -> Union[str, None]: ...
    @property
    def sound_mode_list(self) -> Union[list[str], None]: ...
    @property
    def shuffle(self) -> Union[bool, None]: ...
    @property
    def repeat(self) -> Union[str, None]: ...
    @property
    def group_members(self) -> Union[list[str], None]: ...
    @property
    def supported_features(self) -> int: ...
    def turn_on(self) -> None: ...
    async def async_turn_on(self) -> None: ...
    def turn_off(self) -> None: ...
    async def async_turn_off(self) -> None: ...
    def mute_volume(self, mute) -> None: ...
    async def async_mute_volume(self, mute) -> None: ...
    def set_volume_level(self, volume) -> None: ...
    async def async_set_volume_level(self, volume) -> None: ...
    def media_play(self) -> None: ...
    async def async_media_play(self) -> None: ...
    def media_pause(self) -> None: ...
    async def async_media_pause(self) -> None: ...
    def media_stop(self) -> None: ...
    async def async_media_stop(self) -> None: ...
    def media_previous_track(self) -> None: ...
    async def async_media_previous_track(self) -> None: ...
    def media_next_track(self) -> None: ...
    async def async_media_next_track(self) -> None: ...
    def media_seek(self, position) -> None: ...
    async def async_media_seek(self, position) -> None: ...
    def play_media(self, media_type, media_id, **kwargs) -> None: ...
    async def async_play_media(self, media_type, media_id, **kwargs) -> None: ...
    def select_source(self, source) -> None: ...
    async def async_select_source(self, source) -> None: ...
    def select_sound_mode(self, sound_mode) -> None: ...
    async def async_select_sound_mode(self, sound_mode) -> None: ...
    def clear_playlist(self) -> None: ...
    async def async_clear_playlist(self) -> None: ...
    def set_shuffle(self, shuffle) -> None: ...
    async def async_set_shuffle(self, shuffle) -> None: ...
    def set_repeat(self, repeat) -> None: ...
    async def async_set_repeat(self, repeat) -> None: ...
    @property
    def support_play(self): ...
    @property
    def support_pause(self): ...
    @property
    def support_stop(self): ...
    @property
    def support_seek(self): ...
    @property
    def support_volume_set(self): ...
    @property
    def support_volume_mute(self): ...
    @property
    def support_previous_track(self): ...
    @property
    def support_next_track(self): ...
    @property
    def support_play_media(self): ...
    @property
    def support_select_source(self): ...
    @property
    def support_select_sound_mode(self): ...
    @property
    def support_clear_playlist(self): ...
    @property
    def support_shuffle_set(self): ...
    @property
    def support_grouping(self): ...
    async def async_toggle(self) -> None: ...
    async def async_volume_up(self) -> None: ...
    async def async_volume_down(self) -> None: ...
    async def async_media_play_pause(self) -> None: ...
    @property
    def entity_picture(self): ...
    @property
    def media_image_local(self): ...
    @property
    def capability_attributes(self): ...
    @property
    def state_attributes(self): ...
    async def async_browse_media(self, media_content_type: Union[str, None] = ..., media_content_id: Union[str, None] = ...) -> BrowseMedia: ...
    def join_players(self, group_members) -> None: ...
    async def async_join_players(self, group_members) -> None: ...
    def unjoin_player(self) -> None: ...
    async def async_unjoin_player(self) -> None: ...
    async def _async_fetch_image_from_cache(self, url): ...
    async def _async_fetch_image(self, url): ...
    def get_browse_image_url(self, media_content_type: str, media_content_id: str, media_image_id: Union[str, None] = ...) -> str: ...

class MediaPlayerImageView(HomeAssistantView):
    requires_auth: bool
    url: str
    name: str
    extra_urls: Any
    component: Any
    def __init__(self, component) -> None: ...
    async def get(self, request: web.Request, entity_id: str, media_content_type: Union[str, None] = ..., media_content_id: Union[str, None] = ...) -> web.Response: ...

async def websocket_handle_thumbnail(hass, connection, msg) -> None: ...
async def websocket_browse_media(hass, connection, msg) -> None: ...

class MediaPlayerDevice(MediaPlayerEntity):
    def __init_subclass__(cls, **kwargs) -> None: ...

class BrowseMedia:
    media_class: Any
    media_content_id: Any
    media_content_type: Any
    title: Any
    can_play: Any
    can_expand: Any
    children: Any
    children_media_class: Any
    thumbnail: Any
    def __init__(self, media_class: str, media_content_id: str, media_content_type: str, title: str, can_play: bool, can_expand: bool, *, children: Union[list[BrowseMedia], None] = ..., children_media_class: Union[str, None] = ..., thumbnail: Union[str, None] = ...) -> None: ...
    def as_dict(self, *, parent: bool = ...) -> dict: ...
    def calculate_children_class(self) -> None: ...



from .const import DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .helpers import get_unique_id as get_unique_id
from homeassistant.const import STATE_UNAVAILABLE as STATE_UNAVAILABLE
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.device_registry import DeviceEntry as DeviceEntry
from homeassistant.helpers.entity_registry import EntityRegistry as EntityRegistry, RegistryEntry as RegistryEntry, async_entries_for_device as async_entries_for_device
from typing import Any
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.model.value import Value as ZwaveValue

_LOGGER: Any

class ValueID:
    command_class: str
    endpoint: str
    property_: str
    property_key: Union[str, None]
    @staticmethod
    def from_unique_id(unique_id: str) -> ValueID: ...
    @staticmethod
    def from_string_id(value_id_str: str) -> ValueID: ...
    def is_same_value_different_endpoints(self, other: ValueID) -> bool: ...

def async_migrate_old_entity(hass: HomeAssistant, ent_reg: EntityRegistry, registered_unique_ids: set[str], platform: str, device: DeviceEntry, unique_id: str) -> None: ...
def async_migrate_unique_id(ent_reg: EntityRegistry, platform: str, old_unique_id: str, new_unique_id: str) -> None: ...
def async_migrate_discovered_value(hass: HomeAssistant, ent_reg: EntityRegistry, registered_unique_ids: set[str], device: DeviceEntry, client: ZwaveClient, disc_info: ZwaveDiscoveryInfo) -> None: ...
def get_old_value_ids(value: ZwaveValue) -> list[str]: ...



from .const import DATA_COORDINATOR as DATA_COORDINATOR, DOMAIN as DOMAIN, MANUFACTURER as MANUFACTURER
from .coordinator import CanaryDataUpdateCoordinator as CanaryDataUpdateCoordinator
from .model import SensorTypeItem as SensorTypeItem
from canary.api import Device as Device, Location as Location
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_SIGNAL_STRENGTH as DEVICE_CLASS_SIGNAL_STRENGTH, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, PERCENTAGE as PERCENTAGE, SIGNAL_STRENGTH_DECIBELS_MILLIWATT as SIGNAL_STRENGTH_DECIBELS_MILLIWATT, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any, Final

SENSOR_VALUE_PRECISION: Final[int]
ATTR_AIR_QUALITY: Final[str]
CANARY_PRO: Final[str]
CANARY_FLEX: Final[str]
SENSOR_TYPES: Final[list[SensorTypeItem]]
STATE_AIR_QUALITY_NORMAL: Final[str]
STATE_AIR_QUALITY_ABNORMAL: Final[str]
STATE_AIR_QUALITY_VERY_ABNORMAL: Final[str]

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class CanarySensor(CoordinatorEntity, SensorEntity):
    coordinator: CanaryDataUpdateCoordinator
    _sensor_type: Any
    _device_id: Any
    _attr_name: Any
    _canary_type: Any
    _attr_state: Any
    _attr_unique_id: Any
    _attr_device_info: Any
    _attr_unit_of_measurement: Any
    _attr_device_class: Any
    _attr_icon: Any
    def __init__(self, coordinator: CanaryDataUpdateCoordinator, sensor_type: SensorTypeItem, location: Location, device: Device) -> None: ...
    @property
    def reading(self) -> Union[float, None]: ...
    @property
    def extra_state_attributes(self) -> Union[dict[str, str], None]: ...



from collections.abc import Iterable
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import SERVICE_TOGGLE as SERVICE_TOGGLE, SERVICE_TURN_OFF as SERVICE_TURN_OFF, SERVICE_TURN_ON as SERVICE_TURN_ON, STATE_ON as STATE_ON
from homeassistant.core import HomeAssistant as HomeAssistant, HomeAssistantError as HomeAssistantError, callback as callback
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE, make_entity_service_schema as make_entity_service_schema
from homeassistant.helpers.entity import ToggleEntity as ToggleEntity, ToggleEntityDescription as ToggleEntityDescription
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.loader import bind_hass as bind_hass
from typing import Any

DOMAIN: str
SCAN_INTERVAL: Any
DATA_PROFILES: str
ENTITY_ID_FORMAT: Any
SUPPORT_BRIGHTNESS: int
SUPPORT_COLOR_TEMP: int
SUPPORT_EFFECT: int
SUPPORT_FLASH: int
SUPPORT_COLOR: int
SUPPORT_TRANSITION: int
SUPPORT_WHITE_VALUE: int
ATTR_COLOR_MODE: str
ATTR_SUPPORTED_COLOR_MODES: str
COLOR_MODE_UNKNOWN: str
COLOR_MODE_ONOFF: str
COLOR_MODE_BRIGHTNESS: str
COLOR_MODE_COLOR_TEMP: str
COLOR_MODE_HS: str
COLOR_MODE_XY: str
COLOR_MODE_RGB: str
COLOR_MODE_RGBW: str
COLOR_MODE_RGBWW: str
COLOR_MODE_WHITE: str
VALID_COLOR_MODES: Any
COLOR_MODES_BRIGHTNESS: Any
COLOR_MODES_COLOR: Any

def valid_supported_color_modes(color_modes: Iterable[str]) -> set[str]: ...
def brightness_supported(color_modes: Union[Iterable[str], None]) -> bool: ...
def color_supported(color_modes: Union[Iterable[str], None]) -> bool: ...
def color_temp_supported(color_modes: Union[Iterable[str], None]) -> bool: ...
def get_supported_color_modes(hass: HomeAssistant, entity_id: str) -> Union[set, None]: ...

ATTR_TRANSITION: str
ATTR_RGB_COLOR: str
ATTR_RGBW_COLOR: str
ATTR_RGBWW_COLOR: str
ATTR_XY_COLOR: str
ATTR_HS_COLOR: str
ATTR_COLOR_TEMP: str
ATTR_KELVIN: str
ATTR_MIN_MIREDS: str
ATTR_MAX_MIREDS: str
ATTR_COLOR_NAME: str
ATTR_WHITE_VALUE: str
ATTR_WHITE: str
ATTR_BRIGHTNESS: str
ATTR_BRIGHTNESS_PCT: str
ATTR_BRIGHTNESS_STEP: str
ATTR_BRIGHTNESS_STEP_PCT: str
ATTR_PROFILE: str
ATTR_FLASH: str
FLASH_SHORT: str
FLASH_LONG: str
ATTR_EFFECT_LIST: str
ATTR_EFFECT: str
EFFECT_COLORLOOP: str
EFFECT_RANDOM: str
EFFECT_WHITE: str
COLOR_GROUP: str
LIGHT_PROFILES_FILE: str
VALID_TRANSITION: Any
VALID_BRIGHTNESS: Any
VALID_BRIGHTNESS_PCT: Any
VALID_BRIGHTNESS_STEP: Any
VALID_BRIGHTNESS_STEP_PCT: Any
VALID_FLASH: Any
LIGHT_TURN_ON_SCHEMA: Any
LIGHT_TURN_OFF_SCHEMA: Any
_LOGGER: Any

def is_on(hass: HomeAssistant, entity_id: str) -> bool: ...
def preprocess_turn_on_alternatives(hass, params) -> None: ...
def filter_turn_off_params(light, params): ...
def filter_turn_on_params(light, params): ...
async def async_setup(hass, config): ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
def _coerce_none(value: str) -> None: ...

class Profile:
    name: str
    color_x: Union[float, None]
    color_y: Union[float, None]
    brightness: Union[int, None]
    transition: Union[int, None]
    hs_color: Union[tuple[float, float], None]
    SCHEMA: Any
    def __post_init__(self) -> None: ...
    @classmethod
    def from_csv_row(cls, csv_row: list[str]) -> Profile: ...

class Profiles:
    hass: Any
    data: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    def _load_profile_data(self) -> dict[str, Profile]: ...
    async def async_initialize(self) -> None: ...
    def apply_default(self, entity_id: str, state_on: bool, params: dict) -> None: ...
    def apply_profile(self, name: str, params: dict) -> None: ...

class LightEntityDescription(ToggleEntityDescription): ...

class LightEntity(ToggleEntity):
    entity_description: LightEntityDescription
    _attr_brightness: Union[int, None]
    _attr_color_mode: Union[str, None]
    _attr_color_temp: Union[int, None]
    _attr_effect_list: Union[list[str], None]
    _attr_effect: Union[str, None]
    _attr_hs_color: Union[tuple[float, float], None]
    _attr_max_mireds: int
    _attr_min_mireds: int
    _attr_rgb_color: Union[tuple[int, int, int], None]
    _attr_rgbw_color: Union[tuple[int, int, int, int], None]
    _attr_rgbww_color: Union[tuple[int, int, int, int, int], None]
    _attr_supported_color_modes: Union[set[str], None]
    _attr_supported_features: int
    _attr_xy_color: Union[tuple[float, float], None]
    @property
    def brightness(self) -> Union[int, None]: ...
    @property
    def color_mode(self) -> Union[str, None]: ...
    @property
    def _light_internal_color_mode(self) -> str: ...
    @property
    def hs_color(self) -> Union[tuple[float, float], None]: ...
    @property
    def xy_color(self) -> Union[tuple[float, float], None]: ...
    @property
    def rgb_color(self) -> Union[tuple[int, int, int], None]: ...
    @property
    def rgbw_color(self) -> Union[tuple[int, int, int, int], None]: ...
    @property
    def _light_internal_rgbw_color(self) -> Union[tuple[int, int, int, int], None]: ...
    @property
    def rgbww_color(self) -> Union[tuple[int, int, int, int, int], None]: ...
    @property
    def color_temp(self) -> Union[int, None]: ...
    @property
    def min_mireds(self) -> int: ...
    @property
    def max_mireds(self) -> int: ...
    @property
    def white_value(self) -> Union[int, None]: ...
    @property
    def effect_list(self) -> Union[list[str], None]: ...
    @property
    def effect(self) -> Union[str, None]: ...
    @property
    def capability_attributes(self): ...
    def _light_internal_convert_color(self, color_mode: str) -> dict: ...
    @property
    def state_attributes(self): ...
    @property
    def _light_internal_supported_color_modes(self) -> set: ...
    @property
    def supported_color_modes(self) -> Union[set[str], None]: ...
    @property
    def supported_features(self) -> int: ...

class Light(LightEntity):
    def __init_subclass__(cls, **kwargs) -> None: ...

def legacy_supported_features(supported_features: int, supported_color_modes: Union[list[str], None]) -> int: ...



from . import DOMAIN as DOMAIN, const as const
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_TYPE as CONF_TYPE
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant
from homeassistant.helpers import entity_registry as entity_registry
from typing import Any

ATYP_SET_VALUE: str
ACTION_SCHEMA: Any

async def async_get_actions(hass: HomeAssistant, device_id: str) -> list[dict]: ...
async def async_call_action_from_config(hass: HomeAssistant, config: dict, variables: dict, context: Union[Context, None]) -> None: ...
async def async_get_action_capabilities(hass: HomeAssistant, config: dict) -> dict: ...



from typing import TypedDict

class DeviceComponentDescription(TypedDict):
    path: str
    name: str
    type: str
    default_disabled: bool

class OWServerDeviceDescription(TypedDict):
    path: str
    family: str
    type: str



from .common import FritzBoxBaseEntity as FritzBoxBaseEntity, FritzBoxTools as FritzBoxTools
from .const import DOMAIN as DOMAIN, UPTIME_DEVIATION as UPTIME_DEVIATION
from fritzconnection.lib.fritzstatus import FritzStatus as FritzStatus
from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT, SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import DATA_GIGABYTES as DATA_GIGABYTES, DATA_RATE_KILOBITS_PER_SECOND as DATA_RATE_KILOBITS_PER_SECOND, DATA_RATE_KILOBYTES_PER_SECOND as DATA_RATE_KILOBYTES_PER_SECOND, DEVICE_CLASS_TIMESTAMP as DEVICE_CLASS_TIMESTAMP
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.dt import utcnow as utcnow
from typing import Any, Callable, TypedDict

_LOGGER: Any

def _uptime_calculation(seconds_uptime: float, last_value: Union[str, None]) -> str: ...
def _retrieve_device_uptime_state(status: FritzStatus, last_value: str) -> str: ...
def _retrieve_connection_uptime_state(status: FritzStatus, last_value: Union[str, None]) -> str: ...
def _retrieve_external_ip_state(status: FritzStatus, last_value: str) -> str: ...
def _retrieve_kb_s_sent_state(status: FritzStatus, last_value: str) -> float: ...
def _retrieve_kb_s_received_state(status: FritzStatus, last_value: str) -> float: ...
def _retrieve_max_kb_s_sent_state(status: FritzStatus, last_value: str) -> float: ...
def _retrieve_max_kb_s_received_state(status: FritzStatus, last_value: str) -> float: ...
def _retrieve_gb_sent_state(status: FritzStatus, last_value: str) -> float: ...
def _retrieve_gb_received_state(status: FritzStatus, last_value: str) -> float: ...

class SensorData(TypedDict):
    name: str
    device_class: Union[str, None]
    state_class: Union[str, None]
    last_reset: bool
    unit_of_measurement: Union[str, None]
    icon: Union[str, None]
    state_provider: Callable

SENSOR_DATA: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class FritzBoxSensor(FritzBoxBaseEntity, SensorEntity):
    _sensor_data: Any
    _last_device_value: Any
    _last_wan_value: Any
    _attr_available: bool
    _attr_device_class: Any
    _attr_icon: Any
    _attr_name: Any
    _attr_state_class: Any
    _attr_unit_of_measurement: Any
    _attr_unique_id: Any
    def __init__(self, fritzbox_tools: FritzBoxTools, device_friendly_name: str, sensor_type: str) -> None: ...
    @property
    def _state_provider(self) -> Callable: ...
    _attr_state: Any
    _attr_last_reset: Any
    def update(self) -> None: ...



from .const import DOMAIN as DOMAIN
from homeassistant import config_entries as config_entries
from homeassistant.const import CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers import aiohttp_client as aiohttp_client
from typing import Any

class TileFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    data_schema: Any
    def __init__(self) -> None: ...
    async def _show_form(self, errors: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_import(self, import_config: dict[str, Any]) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



import logging
from .const import CONF_AREA_NAME as CONF_AREA_NAME, CONF_LAT_NE as CONF_LAT_NE, CONF_LAT_SW as CONF_LAT_SW, CONF_LON_NE as CONF_LON_NE, CONF_LON_SW as CONF_LON_SW, CONF_NEW_AREA as CONF_NEW_AREA, CONF_PUBLIC_MODE as CONF_PUBLIC_MODE, CONF_UUID as CONF_UUID, CONF_WEATHER_AREAS as CONF_WEATHER_AREAS, DOMAIN as DOMAIN
from homeassistant import config_entries as config_entries
from homeassistant.const import CONF_SHOW_ON_MAP as CONF_SHOW_ON_MAP
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers import config_entry_oauth2_flow as config_entry_oauth2_flow
from typing import Any

class NetatmoFlowHandler(config_entry_oauth2_flow.AbstractOAuth2FlowHandler):
    DOMAIN: Any
    @staticmethod
    def async_get_options_flow(config_entry: config_entries.ConfigEntry) -> config_entries.OptionsFlow: ...
    @property
    def logger(self) -> logging.Logger: ...
    @property
    def extra_authorize_data(self) -> dict: ...
    async def async_step_user(self, user_input: Union[dict, None] = ...) -> FlowResult: ...

class NetatmoOptionsFlowHandler(config_entries.OptionsFlow):
    config_entry: Any
    options: Any
    def __init__(self, config_entry: config_entries.ConfigEntry) -> None: ...
    async def async_step_init(self, user_input: Union[dict, None] = ...) -> FlowResult: ...
    async def async_step_public_weather_areas(self, user_input: Union[dict, None] = ...) -> FlowResult: ...
    async def async_step_public_weather(self, user_input: dict) -> FlowResult: ...
    def _create_options_entry(self) -> FlowResult: ...

def fix_coordinates(user_input: dict) -> dict: ...



from . import PiHoleEntity as PiHoleEntity
from .const import DATA_KEY_API as DATA_KEY_API, DATA_KEY_COORDINATOR as DATA_KEY_COORDINATOR, SERVICE_DISABLE as SERVICE_DISABLE, SERVICE_DISABLE_ATTR_DURATION as SERVICE_DISABLE_ATTR_DURATION
from homeassistant.components.switch import SwitchEntity as SwitchEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers import entity_platform as entity_platform
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class PiHoleSwitch(PiHoleEntity, SwitchEntity):
    @property
    def name(self) -> str: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def icon(self) -> str: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def async_disable(self, duration: Any = ...) -> None: ...



from .const import DOMAIN as DOMAIN
from .devolo_device import DevoloDeviceEntity as DevoloDeviceEntity
from devolo_home_control_api.devices.zwave import Zwave as Zwave
from devolo_home_control_api.homecontrol import HomeControl as HomeControl
from homeassistant.components.switch import SwitchEntity as SwitchEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class DevoloSwitch(DevoloDeviceEntity, SwitchEntity):
    _binary_switch_property: Any
    _attr_is_on: Any
    def __init__(self, homecontrol: HomeControl, device_instance: Zwave, element_uid: str) -> None: ...
    def turn_on(self, **kwargs: Any) -> None: ...
    def turn_off(self, **kwargs: Any) -> None: ...
    def _sync(self, message: tuple) -> None: ...



import boto3
from .const import AWS_CONF_CONNECT_TIMEOUT as AWS_CONF_CONNECT_TIMEOUT, AWS_CONF_MAX_POOL_CONNECTIONS as AWS_CONF_MAX_POOL_CONNECTIONS, AWS_CONF_READ_TIMEOUT as AWS_CONF_READ_TIMEOUT, CONF_ACCESS_KEY_ID as CONF_ACCESS_KEY_ID, CONF_ENGINE as CONF_ENGINE, CONF_OUTPUT_FORMAT as CONF_OUTPUT_FORMAT, CONF_REGION as CONF_REGION, CONF_SAMPLE_RATE as CONF_SAMPLE_RATE, CONF_SECRET_ACCESS_KEY as CONF_SECRET_ACCESS_KEY, CONF_TEXT_TYPE as CONF_TEXT_TYPE, CONF_VOICE as CONF_VOICE, CONTENT_TYPE_EXTENSIONS as CONTENT_TYPE_EXTENSIONS, DEFAULT_ENGINE as DEFAULT_ENGINE, DEFAULT_OUTPUT_FORMAT as DEFAULT_OUTPUT_FORMAT, DEFAULT_REGION as DEFAULT_REGION, DEFAULT_SAMPLE_RATES as DEFAULT_SAMPLE_RATES, DEFAULT_TEXT_TYPE as DEFAULT_TEXT_TYPE, DEFAULT_VOICE as DEFAULT_VOICE, SUPPORTED_ENGINES as SUPPORTED_ENGINES, SUPPORTED_OUTPUT_FORMATS as SUPPORTED_OUTPUT_FORMATS, SUPPORTED_REGIONS as SUPPORTED_REGIONS, SUPPORTED_SAMPLE_RATES as SUPPORTED_SAMPLE_RATES, SUPPORTED_SAMPLE_RATES_MAP as SUPPORTED_SAMPLE_RATES_MAP, SUPPORTED_TEXT_TYPES as SUPPORTED_TEXT_TYPES, SUPPORTED_VOICES as SUPPORTED_VOICES
from homeassistant.components.tts import Provider as Provider, TtsAudioType as TtsAudioType
from homeassistant.const import ATTR_CREDENTIALS as ATTR_CREDENTIALS, CONF_PROFILE_NAME as CONF_PROFILE_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any, Final

_LOGGER: Final[Any]
PLATFORM_SCHEMA: Final[Any]

def get_engine(hass: HomeAssistant, config: ConfigType, discovery_info: Union[DiscoveryInfoType, None] = ...) -> Union[Provider, None]: ...

class AmazonPollyProvider(Provider):
    client: Any
    config: Any
    supported_langs: Any
    all_voices: Any
    default_voice: Any
    name: str
    def __init__(self, polly_client: boto3.client, config: ConfigType, supported_languages: list[str], all_voices: dict[str, dict[str, str]]) -> None: ...
    @property
    def supported_languages(self) -> list[str]: ...
    @property
    def default_language(self) -> Union[str, None]: ...
    @property
    def default_options(self) -> dict[str, str]: ...
    @property
    def supported_options(self) -> list[str]: ...
    def get_tts_audio(self, message: str, language: Union[str, None] = ..., options: Union[dict[str, str], None] = ...) -> TtsAudioType: ...



import abc
from . import BondHub as BondHub
from .const import BPUP_SUBS as BPUP_SUBS, DOMAIN as DOMAIN, HUB as HUB
from .entity import BondEntity as BondEntity
from .utils import BondDevice as BondDevice
from bond_api import BPUPSubscriptions as BPUPSubscriptions
from homeassistant.components.light import ATTR_BRIGHTNESS as ATTR_BRIGHTNESS, LightEntity as LightEntity, SUPPORT_BRIGHTNESS as SUPPORT_BRIGHTNESS
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class BondBaseLight(BondEntity, LightEntity, metaclass=abc.ABCMeta):
    _attr_supported_features: int

class BondLight(BondBaseLight, BondEntity, LightEntity):
    _attr_supported_features: Any
    def __init__(self, hub: BondHub, device: BondDevice, bpup_subs: BPUPSubscriptions, sub_device: Union[str, None] = ...) -> None: ...
    _attr_is_on: Any
    _attr_brightness: Any
    def _apply_state(self, state: dict) -> None: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...

class BondDownLight(BondBaseLight, BondEntity, LightEntity):
    _attr_is_on: Any
    def _apply_state(self, state: dict) -> None: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...

class BondUpLight(BondBaseLight, BondEntity, LightEntity):
    _attr_is_on: Any
    def _apply_state(self, state: dict) -> None: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...

class BondFireplace(BondEntity, LightEntity):
    _attr_supported_features: Any
    _attr_is_on: Any
    _attr_brightness: Any
    _attr_icon: Any
    def _apply_state(self, state: dict) -> None: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...



from .const import CONF_RESPOND_TO_READ as CONF_RESPOND_TO_READ, CONF_STATE_ADDRESS as CONF_STATE_ADDRESS, CONF_SYNC_STATE as CONF_SYNC_STATE, DOMAIN as DOMAIN, KNX_ADDRESS as KNX_ADDRESS
from .knx_entity import KnxEntity as KnxEntity
from .schema import SelectSchema as SelectSchema
from homeassistant.components.select import SelectEntity as SelectEntity
from homeassistant.const import CONF_NAME as CONF_NAME, STATE_UNAVAILABLE as STATE_UNAVAILABLE, STATE_UNKNOWN as STATE_UNKNOWN
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.restore_state import RestoreEntity as RestoreEntity
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any
from xknx import XKNX as XKNX
from xknx.devices import Device as XknxDevice, RawValue

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...
def _create_raw_value(xknx: XKNX, config: ConfigType) -> RawValue: ...

class KNXSelect(KnxEntity, SelectEntity, RestoreEntity):
    _device: RawValue
    _option_payloads: Any
    _attr_options: Any
    _attr_current_option: Any
    _attr_unique_id: Any
    def __init__(self, xknx: XKNX, config: ConfigType) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def after_update_callback(self, device: XknxDevice) -> None: ...
    def option_from_payload(self, payload: Union[int, None]) -> Union[str, None]: ...
    async def async_select_option(self, option: str) -> None: ...



import aioshelly
from . import ShellyDeviceWrapper as ShellyDeviceWrapper
from .const import LAST_RESET_NEVER as LAST_RESET_NEVER, LAST_RESET_UPTIME as LAST_RESET_UPTIME, SHAIR_MAX_WORK_HOURS as SHAIR_MAX_WORK_HOURS
from .entity import BlockAttributeDescription as BlockAttributeDescription, RestAttributeDescription as RestAttributeDescription, ShellyBlockAttributeEntity as ShellyBlockAttributeEntity, ShellyRestAttributeEntity as ShellyRestAttributeEntity, ShellySleepingBlockAttributeEntity as ShellySleepingBlockAttributeEntity, async_setup_entry_attribute_entities as async_setup_entry_attribute_entities, async_setup_entry_rest as async_setup_entry_rest
from .utils import get_device_uptime as get_device_uptime, temperature_unit as temperature_unit
from homeassistant.components import sensor as sensor
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONCENTRATION_PARTS_PER_MILLION as CONCENTRATION_PARTS_PER_MILLION, DEGREE as DEGREE, ELECTRIC_CURRENT_AMPERE as ELECTRIC_CURRENT_AMPERE, ELECTRIC_POTENTIAL_VOLT as ELECTRIC_POTENTIAL_VOLT, ENERGY_KILO_WATT_HOUR as ENERGY_KILO_WATT_HOUR, LIGHT_LUX as LIGHT_LUX, PERCENTAGE as PERCENTAGE, POWER_WATT as POWER_WATT, SIGNAL_STRENGTH_DECIBELS_MILLIWATT as SIGNAL_STRENGTH_DECIBELS_MILLIWATT
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import StateType as StateType
from homeassistant.util import dt as dt
from typing import Any, Final

_LOGGER: Final[Any]
SENSORS: Final[Any]
REST_SENSORS: Final[Any]

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class ShellySensor(ShellyBlockAttributeEntity, SensorEntity):
    _last_value: Any
    _attr_last_reset: Any
    def __init__(self, wrapper: ShellyDeviceWrapper, block: aioshelly.Block, attribute: str, description: BlockAttributeDescription) -> None: ...
    @property
    def state(self) -> StateType: ...
    @property
    def state_class(self) -> Union[str, None]: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...

class ShellyRestSensor(ShellyRestAttributeEntity, SensorEntity):
    @property
    def state(self) -> StateType: ...
    @property
    def state_class(self) -> Union[str, None]: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...

class ShellySleepingSensor(ShellySleepingBlockAttributeEntity, SensorEntity):
    @property
    def state(self) -> StateType: ...
    @property
    def state_class(self) -> Union[str, None]: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...



from .const import CONF_USE_NEAREST as CONF_USE_NEAREST, DOMAIN as DOMAIN, NO_AIRLY_SENSORS as NO_AIRLY_SENSORS
from aiohttp import ClientSession as ClientSession
from homeassistant import config_entries as config_entries
from homeassistant.const import CONF_API_KEY as CONF_API_KEY, CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE, CONF_NAME as CONF_NAME, HTTP_NOT_FOUND as HTTP_NOT_FOUND, HTTP_UNAUTHORIZED as HTTP_UNAUTHORIZED
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from typing import Any

class AirlyFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...

async def test_location(client: ClientSession, api_key: str, latitude: float, longitude: float, use_nearest: bool = ...) -> bool: ...



from datetime import datetime
from homeassistant.const import EVENT_HOMEASSISTANT_START as EVENT_HOMEASSISTANT_START, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import CoreState as CoreState, HomeAssistant as HomeAssistant, State as State, callback as callback, valid_entity_id as valid_entity_id
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.event import async_track_time_interval as async_track_time_interval
from homeassistant.helpers.json import JSONEncoder as JSONEncoder
from homeassistant.helpers.singleton import singleton as singleton
from homeassistant.helpers.storage import Store as Store
from typing import Any

DATA_RESTORE_STATE_TASK: str
_LOGGER: Any
STORAGE_KEY: str
STORAGE_VERSION: int
STATE_DUMP_INTERVAL: Any
STATE_EXPIRATION: Any

class StoredState:
    state: Any
    last_seen: Any
    def __init__(self, state: State, last_seen: datetime) -> None: ...
    def as_dict(self) -> dict[str, Any]: ...
    @classmethod
    def from_dict(cls, json_dict: dict) -> StoredState: ...

class RestoreStateData:
    @classmethod
    async def async_get_instance(cls, hass: HomeAssistant) -> RestoreStateData: ...
    hass: Any
    store: Any
    last_states: Any
    entity_ids: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    def async_get_stored_states(self) -> list[StoredState]: ...
    async def async_dump_states(self) -> None: ...
    def async_setup_dump(self, *args: Any) -> None: ...
    def async_restore_entity_added(self, entity_id: str) -> None: ...
    def async_restore_entity_removed(self, entity_id: str) -> None: ...

def _encode(value: Any) -> Any: ...
def _encode_complex(value: Any) -> Any: ...

class RestoreEntity(Entity):
    async def async_internal_added_to_hass(self) -> None: ...
    async def async_internal_will_remove_from_hass(self) -> None: ...
    async def async_get_last_state(self) -> Union[State, None]: ...



from .view import HomeAssistantView as HomeAssistantView
from aiohttp.web import Application as Application, Request as Request, StreamResponse as StreamResponse
from collections.abc import Awaitable, Callable as Callable
from datetime import datetime
from homeassistant.config import load_yaml_config_file as load_yaml_config_file
from homeassistant.const import HTTP_BAD_REQUEST as HTTP_BAD_REQUEST
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.util import yaml as yaml
from typing import Any, Final

_LOGGER: Final[Any]
KEY_BANNED_IPS: Final[str]
KEY_FAILED_LOGIN_ATTEMPTS: Final[str]
KEY_LOGIN_THRESHOLD: Final[str]
NOTIFICATION_ID_BAN: Final[str]
NOTIFICATION_ID_LOGIN: Final[str]
IP_BANS_FILE: Final[str]
ATTR_BANNED_AT: Final[str]
SCHEMA_IP_BAN_ENTRY: Final[Any]

def setup_bans(hass: HomeAssistant, app: Application, login_threshold: int) -> None: ...
async def ban_middleware(request: Request, handler: Callable[[Request], Awaitable[StreamResponse]]) -> StreamResponse: ...
def log_invalid_auth(func: Callable[..., Awaitable[StreamResponse]]) -> Callable[..., Awaitable[StreamResponse]]: ...
async def process_wrong_login(request: Request) -> None: ...
async def process_success_login(request: Request) -> None: ...

class IpBan:
    ip_address: Any
    banned_at: Any
    def __init__(self, ip_ban: str, banned_at: Union[datetime, None] = ...) -> None: ...

async def async_load_ip_bans_config(hass: HomeAssistant, path: str) -> list[IpBan]: ...
def update_ip_bans_config(path: str, ip_ban: IpBan) -> None: ...



import asyncio
from aioesphomeapi import APIClient as APIClient, APIVersion, DeviceInfo, EntityInfo as EntityInfo, EntityState as EntityState, UserService
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.storage import Store as Store
from typing import Any, Callable

SAVE_DELAY: int
INFO_TYPE_TO_PLATFORM: dict[type[EntityInfo], str]

class RuntimeEntryData:
    entry_id: str
    client: APIClient
    store: Store
    state: dict[str, dict[int, EntityState]]
    info: dict[str, dict[int, EntityInfo]]
    old_info: dict[str, dict[int, EntityInfo]]
    services: dict[int, UserService]
    available: bool
    device_info: Union[DeviceInfo, None]
    api_version: APIVersion
    cleanup_callbacks: list[Callable[[], None]]
    disconnect_callbacks: list[Callable[[], None]]
    loaded_platforms: set[str]
    platform_load_lock: asyncio.Lock
    _storage_contents: Union[dict[str, Any], None]
    def async_update_entity(self, hass: HomeAssistant, component_key: str, key: int) -> None: ...
    def async_remove_entity(self, hass: HomeAssistant, component_key: str, key: int) -> None: ...
    async def _ensure_platforms_loaded(self, hass: HomeAssistant, entry: ConfigEntry, platforms: set[str]) -> None: ...
    async def async_update_static_infos(self, hass: HomeAssistant, entry: ConfigEntry, infos: list[EntityInfo]) -> None: ...
    def async_update_state(self, hass: HomeAssistant, state: EntityState) -> None: ...
    def async_update_device_state(self, hass: HomeAssistant) -> None: ...
    async def async_load_from_store(self) -> tuple[list[EntityInfo], list[UserService]]: ...
    async def async_save_to_store(self) -> None: ...



from . import DOMAIN as DOMAIN, SUPPORT_OPEN as SUPPORT_OPEN
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_TYPE as CONF_TYPE, SERVICE_LOCK as SERVICE_LOCK, SERVICE_OPEN as SERVICE_OPEN, SERVICE_UNLOCK as SERVICE_UNLOCK
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.entity import get_supported_features as get_supported_features
from typing import Any

ACTION_TYPES: Any
ACTION_SCHEMA: Any

async def async_get_actions(hass: HomeAssistant, device_id: str) -> list[dict]: ...
async def async_call_action_from_config(hass: HomeAssistant, config: dict, variables: dict, context: Union[Context, None]) -> None: ...



from homeassistant.components.light import ATTR_TRANSITION as ATTR_TRANSITION
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_PLATFORM as CONF_PLATFORM, SERVICE_TURN_ON as SERVICE_TURN_ON
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from typing import Any

DOMAIN: str
STATE: str
STATES: str

def _hass_domain_validator(config): ...
def _platform_validator(config): ...

PLATFORM_SCHEMA: Any

async def async_setup(hass, config): ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class Scene(Entity):
    @property
    def should_poll(self) -> bool: ...
    @property
    def state(self) -> Union[str, None]: ...
    def activate(self, **kwargs: Any) -> None: ...
    async def async_activate(self, **kwargs: Any) -> None: ...



from PIL import ImageDraw as ImageDraw

def draw_box(draw: ImageDraw, box: tuple[float, float, float, float], img_width: int, img_height: int, text: str = ..., color: tuple[int, int, int] = ...) -> None: ...



from .config_flow import get_client_controller as get_client_controller
from .const import CONF_ZONE_RUN_TIME as CONF_ZONE_RUN_TIME, DATA_CONTROLLER as DATA_CONTROLLER, DATA_COORDINATOR as DATA_COORDINATOR, DATA_PROGRAMS as DATA_PROGRAMS, DATA_PROVISION_SETTINGS as DATA_PROVISION_SETTINGS, DATA_RESTRICTIONS_CURRENT as DATA_RESTRICTIONS_CURRENT, DATA_RESTRICTIONS_UNIVERSAL as DATA_RESTRICTIONS_UNIVERSAL, DATA_ZONES as DATA_ZONES, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_ATTRIBUTION as ATTR_ATTRIBUTION, CONF_IP_ADDRESS as CONF_IP_ADDRESS, CONF_PASSWORD as CONF_PASSWORD, CONF_PORT as CONF_PORT, CONF_SSL as CONF_SSL
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity, DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from homeassistant.util.network import is_ip_address as is_ip_address
from regenmaschine.controller import Controller as Controller
from typing import Any

DEFAULT_ATTRIBUTION: str
DEFAULT_ICON: str
DEFAULT_SSL: bool
DEFAULT_UPDATE_INTERVAL: Any
CONFIG_SCHEMA: Any
PLATFORMS: Any

async def async_update_programs_and_zones(hass: HomeAssistant, entry: ConfigEntry) -> None: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_reload_entry(hass: HomeAssistant, entry: ConfigEntry) -> None: ...

class RainMachineEntity(CoordinatorEntity):
    _attr_device_info: Any
    _attr_extra_state_attributes: Any
    _attr_unique_id: Any
    _controller: Any
    _entity_type: Any
    def __init__(self, coordinator: DataUpdateCoordinator, controller: Controller, entity_type: str) -> None: ...
    def _handle_coordinator_update(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def update_from_latest_data(self) -> None: ...



from typing import Any, Final

DOMAIN: Final[str]
STORAGE_KEY: Final[str]
STORAGE_VERSION: Final[int]
ATTR_ADAPTERS: Final[str]
ATTR_CONFIGURED_ADAPTERS: Final[str]
DEFAULT_CONFIGURED_ADAPTERS: list[str]
MDNS_TARGET_IP: Final[str]
PUBLIC_TARGET_IP: Final[str]
NETWORK_CONFIG_SCHEMA: Any



from typing import Any
from zeroconf import DNSRecord as DNSRecord, Zeroconf
from zeroconf.asyncio import AsyncServiceBrowser, AsyncZeroconf

TYPE_AAAA: int

class HaZeroconf(Zeroconf):
    def close(self) -> None: ...
    ha_close: Any

class HaAsyncZeroconf(AsyncZeroconf):
    async def async_close(self) -> None: ...
    ha_async_close: Any

class HaAsyncServiceBrowser(AsyncServiceBrowser):
    ipv6: Any
    def __init__(self, ipv6: bool, *args: Any, **kwargs: Any) -> None: ...
    def update_record(self, zc: Zeroconf, now: float, record: DNSRecord) -> None: ...



from collections.abc import Container, Iterable
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.util import slugify as slugify
from typing import Any

DATA_REGISTRY: str
EVENT_AREA_REGISTRY_UPDATED: str
STORAGE_KEY: str
STORAGE_VERSION: int
SAVE_DELAY: int

class AreaEntry:
    name: str
    normalized_name: str
    id: Union[str, None]
    def generate_id(self, existing_ids: Container[str]) -> None: ...
    def __init__(self, name, normalized_name, id) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class AreaRegistry:
    hass: Any
    areas: Any
    _store: Any
    _normalized_name_area_idx: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    def async_get_area(self, area_id: str) -> Union[AreaEntry, None]: ...
    def async_get_area_by_name(self, name: str) -> Union[AreaEntry, None]: ...
    def async_list_areas(self) -> Iterable[AreaEntry]: ...
    def async_get_or_create(self, name: str) -> AreaEntry: ...
    def async_create(self, name: str) -> AreaEntry: ...
    def async_delete(self, area_id: str) -> None: ...
    def async_update(self, area_id: str, name: str) -> AreaEntry: ...
    def _async_update(self, area_id: str, name: str) -> AreaEntry: ...
    async def async_load(self) -> None: ...
    def async_schedule_save(self) -> None: ...
    def _data_to_save(self) -> dict[str, list[dict[str, Union[str, None]]]]: ...

def async_get(hass: HomeAssistant) -> AreaRegistry: ...
async def async_load(hass: HomeAssistant) -> None: ...
async def async_get_registry(hass: HomeAssistant) -> AreaRegistry: ...
def normalize_area_name(area_name: str) -> str: ...



import voluptuous as vol
from . import MULTI_FACTOR_AUTH_MODULES as MULTI_FACTOR_AUTH_MODULES, MULTI_FACTOR_AUTH_MODULE_SCHEMA as MULTI_FACTOR_AUTH_MODULE_SCHEMA, MultiFactorAuthModule as MultiFactorAuthModule, SetupFlow as SetupFlow
from homeassistant.core import HomeAssistant as HomeAssistant
from typing import Any

CONFIG_SCHEMA: Any

class InsecureExampleModule(MultiFactorAuthModule):
    DEFAULT_TITLE: str
    _data: Any
    def __init__(self, hass: HomeAssistant, config: dict[str, Any]) -> None: ...
    @property
    def input_schema(self) -> vol.Schema: ...
    @property
    def setup_schema(self) -> vol.Schema: ...
    async def async_setup_flow(self, user_id: str) -> SetupFlow: ...
    async def async_setup_user(self, user_id: str, setup_data: Any) -> Any: ...
    async def async_depose_user(self, user_id: str) -> None: ...
    async def async_is_user_setup(self, user_id: str) -> bool: ...
    async def async_validate(self, user_id: str, user_input: dict[str, Any]) -> bool: ...



from .renault_vehicle import RenaultVehicleProxy as RenaultVehicleProxy
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from homeassistant.util import slugify as slugify
from renault_api.kamereon.models import KamereonVehicleBatteryStatusData, KamereonVehicleChargeModeData, KamereonVehicleCockpitData, KamereonVehicleHvacStatusData
from typing import Any, Optional, TypeVar

ATTR_LAST_UPDATE: str
T = TypeVar('T')

class RenaultDataEntity(CoordinatorEntity[Optional[T]], Entity):
    vehicle: Any
    _entity_type: Any
    _attr_device_info: Any
    _attr_name: Any
    _attr_unique_id: Any
    def __init__(self, vehicle: RenaultVehicleProxy, entity_type: str, coordinator_key: str) -> None: ...
    @property
    def available(self) -> bool: ...
    @property
    def data(self) -> Union[T, None]: ...

class RenaultBatteryDataEntity(RenaultDataEntity[KamereonVehicleBatteryStatusData]):
    def __init__(self, vehicle: RenaultVehicleProxy, entity_type: str) -> None: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...
    @property
    def is_charging(self) -> bool: ...
    @property
    def is_plugged_in(self) -> bool: ...

class RenaultChargeModeDataEntity(RenaultDataEntity[KamereonVehicleChargeModeData]):
    def __init__(self, vehicle: RenaultVehicleProxy, entity_type: str) -> None: ...

class RenaultCockpitDataEntity(RenaultDataEntity[KamereonVehicleCockpitData]):
    def __init__(self, vehicle: RenaultVehicleProxy, entity_type: str) -> None: ...

class RenaultHVACDataEntity(RenaultDataEntity[KamereonVehicleHvacStatusData]):
    def __init__(self, vehicle: RenaultVehicleProxy, entity_type: str) -> None: ...



from .const import LEASES_REGEX as LEASES_REGEX
from .model import Device as Device
from homeassistant.components.device_tracker import DOMAIN as DOMAIN, DeviceScanner as DeviceScanner
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Final

_LOGGER: Final[Any]
PLATFORM_SCHEMA: Final[Any]

def get_scanner(hass: HomeAssistant, config: ConfigType) -> Union[DeviceScanner, None]: ...

class ActiontecDeviceScanner(DeviceScanner):
    host: Any
    username: Any
    password: Any
    last_results: Any
    success_init: Any
    def __init__(self, config: ConfigType) -> None: ...
    def scan_devices(self) -> list[str]: ...
    def get_device_name(self, device: str) -> Union[str, None]: ...
    def _update_info(self) -> bool: ...
    def get_actiontec_data(self) -> Union[list[Device], None]: ...



from typing import TypedDict

class IPv6ConfiguredAddress(TypedDict):
    address: str
    flowinfo: int
    scope_id: int
    network_prefix: int

class IPv4ConfiguredAddress(TypedDict):
    address: str
    network_prefix: int

class Adapter(TypedDict):
    name: str
    enabled: bool
    auto: bool
    default: bool
    ipv6: list[IPv6ConfiguredAddress]
    ipv4: list[IPv4ConfiguredAddress]



from . import Recorder as Recorder
from .const import DOMAIN as DOMAIN
from .models import StatisticMetaData as StatisticMetaData, Statistics as Statistics, StatisticsMeta as StatisticsMeta, process_timestamp_to_utc_isoformat as process_timestamp_to_utc_isoformat
from .util import execute as execute, retryable_database_job as retryable_database_job, session_scope as session_scope
from datetime import datetime
from homeassistant.const import PRESSURE_PA as PRESSURE_PA, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.util.unit_system import UnitSystem as UnitSystem
from sqlalchemy.orm.scoping import scoped_session as scoped_session
from typing import Any

QUERY_STATISTICS: Any
QUERY_STATISTIC_META: Any
STATISTICS_BAKERY: str
STATISTICS_META_BAKERY: str
UNIT_CONVERSIONS: Any
_LOGGER: Any

def async_setup(hass: HomeAssistant) -> None: ...
def get_start_time() -> datetime: ...
def _get_metadata_ids(hass: HomeAssistant, session: scoped_session, statistic_ids: list[str]) -> list[str]: ...
def _get_or_add_metadata_id(hass: HomeAssistant, session: scoped_session, statistic_id: str, metadata: StatisticMetaData) -> str: ...
def compile_statistics(instance: Recorder, start: datetime) -> bool: ...
def _get_metadata(hass: HomeAssistant, session: scoped_session, statistic_ids: Union[list[str], None], statistic_type: Union[str, None]) -> dict[str, dict[str, str]]: ...
def _configured_unit(unit: str, units: UnitSystem) -> str: ...
def list_statistic_ids(hass: HomeAssistant, statistic_type: Union[str, None] = ...) -> list[Union[dict[str, str], None]]: ...
def statistics_during_period(hass: HomeAssistant, start_time: datetime, end_time: Union[datetime, None] = ..., statistic_ids: Union[list[str], None] = ...) -> dict[str, list[dict[str, str]]]: ...
def get_last_statistics(hass: HomeAssistant, number_of_stats: int, statistic_id: str) -> dict[str, list[dict]]: ...
def _sorted_statistics_to_dict(hass: HomeAssistant, stats: list, statistic_ids: Union[list[str], None], metadata: dict[str, dict[str, str]]) -> dict[str, list[dict]]: ...



from homeassistant.components import http as http
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.util.network import is_ip_address as is_ip_address, is_loopback as is_loopback, normalize_url as normalize_url

TYPE_URL_INTERNAL: str
TYPE_URL_EXTERNAL: str

class NoURLAvailableError(HomeAssistantError): ...

def is_internal_request(hass: HomeAssistant) -> bool: ...
def get_url(hass: HomeAssistant, *, require_current_request: bool = ..., require_ssl: bool = ..., require_standard_port: bool = ..., allow_internal: bool = ..., allow_external: bool = ..., allow_cloud: bool = ..., allow_ip: bool = ..., prefer_external: bool = ..., prefer_cloud: bool = ...) -> str: ...
def _get_request_host() -> Union[str, None]: ...
def _get_internal_url(hass: HomeAssistant, *, allow_ip: bool = ..., require_current_request: bool = ..., require_ssl: bool = ..., require_standard_port: bool = ...) -> str: ...
def _get_external_url(hass: HomeAssistant, *, allow_cloud: bool = ..., allow_ip: bool = ..., prefer_cloud: bool = ..., require_current_request: bool = ..., require_ssl: bool = ..., require_standard_port: bool = ...) -> str: ...
def _get_cloud_url(hass: HomeAssistant, require_current_request: bool = ...) -> str: ...



from homeassistant.const import PRESSURE as PRESSURE, PRESSURE_HPA as PRESSURE_HPA, PRESSURE_INHG as PRESSURE_INHG, PRESSURE_MBAR as PRESSURE_MBAR, PRESSURE_PA as PRESSURE_PA, PRESSURE_PSI as PRESSURE_PSI, UNIT_NOT_RECOGNIZED_TEMPLATE as UNIT_NOT_RECOGNIZED_TEMPLATE

VALID_UNITS: tuple[str, ...]
UNIT_CONVERSION: dict[str, float]

def convert(value: float, unit_1: str, unit_2: str) -> float: ...



from collections.abc import Mapping
from homeassistant.components import websocket_api as websocket_api
from homeassistant.const import ATTR_FRIENDLY_NAME as ATTR_FRIENDLY_NAME
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import TemplateError as TemplateError
from homeassistant.helpers.entity import async_generate_entity_id as async_generate_entity_id
from homeassistant.helpers.template import Template as Template
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.util import slugify as slugify
from typing import Any

ATTR_CREATED_AT: str
ATTR_MESSAGE: str
ATTR_NOTIFICATION_ID: str
ATTR_TITLE: str
ATTR_STATUS: str
DOMAIN: str
ENTITY_ID_FORMAT: Any
EVENT_PERSISTENT_NOTIFICATIONS_UPDATED: str
SERVICE_CREATE: str
SERVICE_DISMISS: str
SERVICE_MARK_READ: str
SCHEMA_SERVICE_CREATE: Any
SCHEMA_SERVICE_DISMISS: Any
SCHEMA_SERVICE_MARK_READ: Any
DEFAULT_OBJECT_ID: str
_LOGGER: Any
STATE: str
STATUS_UNREAD: str
STATUS_READ: str

def create(hass, message, title: Any | None = ..., notification_id: Any | None = ...) -> None: ...
def dismiss(hass, notification_id) -> None: ...
def async_create(hass: HomeAssistant, message: str, title: Union[str, None] = ..., notification_id: Union[str, None] = ...) -> None: ...
def async_dismiss(hass: HomeAssistant, notification_id: str) -> None: ...
async def async_setup(hass: HomeAssistant, config: dict) -> bool: ...
def websocket_get_notifications(hass: HomeAssistant, connection: websocket_api.ActiveConnection, msg: Mapping[str, Any]) -> None: ...



from .const import CONF_PLACE_ID as CONF_PLACE_ID, CONF_SERVICE_ID as CONF_SERVICE_ID, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant import config_entries as config_entries
from homeassistant.const import CONF_FRIENDLY_NAME as CONF_FRIENDLY_NAME
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers import aiohttp_client as aiohttp_client
from typing import Any

DATA_SCHEMA: Any

class ConfigFlow(config_entries.ConfigFlow):
    VERSION: int
    @staticmethod
    def async_get_options_flow(config_entry: config_entries.ConfigEntry) -> config_entries.OptionsFlow: ...
    async def async_step_import(self, import_config: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...

class RecollectWasteOptionsFlowHandler(config_entries.OptionsFlow):
    _entry: Any
    def __init__(self, entry: config_entries.ConfigEntry) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



from .core import Context as Context
from collections.abc import Generator, Sequence
from typing import Any

class HomeAssistantError(Exception): ...
class InvalidEntityFormatError(HomeAssistantError): ...
class NoEntitySpecifiedError(HomeAssistantError): ...

class TemplateError(HomeAssistantError):
    def __init__(self, exception: Exception) -> None: ...

class ConditionError(HomeAssistantError):
    type: str
    @staticmethod
    def _indent(indent: int, message: str) -> str: ...
    def output(self, indent: int) -> Generator: ...
    def __str__(self) -> str: ...
    def __init__(self, type) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class ConditionErrorMessage(ConditionError):
    message: str
    def output(self, indent: int) -> Generator: ...
    def __init__(self, type, message) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class ConditionErrorIndex(ConditionError):
    index: int
    total: int
    error: ConditionError
    def output(self, indent: int) -> Generator: ...
    def __init__(self, type, index, total, error) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class ConditionErrorContainer(ConditionError):
    errors: Sequence[ConditionError]
    def output(self, indent: int) -> Generator: ...
    def __init__(self, type, errors) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class IntegrationError(HomeAssistantError):
    def __str__(self) -> str: ...

class PlatformNotReady(IntegrationError): ...
class ConfigEntryNotReady(IntegrationError): ...
class ConfigEntryAuthFailed(IntegrationError): ...
class InvalidStateError(HomeAssistantError): ...

class Unauthorized(HomeAssistantError):
    context: Any
    user_id: Any
    entity_id: Any
    config_entry_id: Any
    perm_category: Any
    permission: Any
    def __init__(self, context: Union[Context, None] = ..., user_id: Union[str, None] = ..., entity_id: Union[str, None] = ..., config_entry_id: Union[str, None] = ..., perm_category: Union[str, None] = ..., permission: Union[str, None] = ...) -> None: ...

class UnknownUser(Unauthorized): ...

class ServiceNotFound(HomeAssistantError):
    domain: Any
    service: Any
    def __init__(self, domain: str, service: str) -> None: ...
    def __str__(self) -> str: ...

class MaxLengthExceeded(HomeAssistantError):
    value: Any
    property_name: Any
    max_length: Any
    def __init__(self, value: str, property_name: str, max_length: int) -> None: ...

class RequiredParameterMissing(HomeAssistantError):
    parameter_names: Any
    def __init__(self, parameter_names: list[str]) -> None: ...



from .util import etree_to_dict as etree_to_dict
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from typing import Any

_LOGGER: Any

class DescriptionManager:
    hass: Any
    _description_cache: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    async def fetch_description(self, xml_location: Union[str, None]) -> Union[None, dict[str, str]]: ...
    def async_cached_description(self, xml_location: str) -> Union[None, dict[str, str]]: ...
    async def _fetch_description(self, xml_location: str) -> Union[None, dict[str, str]]: ...



import abc
import logging
from abc import ABC, abstractmethod
from homeassistant.components import websocket_api as websocket_api
from homeassistant.const import CONF_ID as CONF_ID
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.storage import Store as Store
from homeassistant.util import slugify as slugify
from typing import Any, Awaitable, Callable, Iterable

STORAGE_VERSION: int
SAVE_DELAY: int
CHANGE_ADDED: str
CHANGE_UPDATED: str
CHANGE_REMOVED: str

class CollectionChangeSet:
    change_type: str
    item_id: str
    item: Any
ChangeListener = Callable[[str, str, dict], Awaitable[None]]
ChangeSetListener = Callable[[Iterable[CollectionChangeSet]], Awaitable[None]]

class CollectionError(HomeAssistantError): ...

class ItemNotFound(CollectionError):
    item_id: Any
    def __init__(self, item_id: str) -> None: ...

class IDManager:
    collections: Any
    def __init__(self) -> None: ...
    def add_collection(self, collection: dict[str, Any]) -> None: ...
    def has_id(self, item_id: str) -> bool: ...
    def generate_id(self, suggestion: str) -> str: ...

class ObservableCollection(ABC):
    logger: Any
    id_manager: Any
    data: Any
    listeners: Any
    change_set_listeners: Any
    def __init__(self, logger: logging.Logger, id_manager: Union[IDManager, None] = ...) -> None: ...
    def async_items(self) -> list[dict]: ...
    def async_add_listener(self, listener: ChangeListener) -> None: ...
    def async_add_change_set_listener(self, listener: ChangeSetListener) -> None: ...
    async def notify_changes(self, change_sets: Iterable[CollectionChangeSet]) -> None: ...

class YamlCollection(ObservableCollection):
    async def async_load(self, data: list[dict]) -> None: ...

class StorageCollection(ObservableCollection, metaclass=abc.ABCMeta):
    store: Any
    def __init__(self, store: Store, logger: logging.Logger, id_manager: Union[IDManager, None] = ...) -> None: ...
    @property
    def hass(self) -> HomeAssistant: ...
    async def _async_load_data(self) -> Union[dict, None]: ...
    async def async_load(self) -> None: ...
    @abstractmethod
    async def _process_create_data(self, data: dict) -> dict: ...
    @abstractmethod
    def _get_suggested_id(self, info: dict) -> str: ...
    @abstractmethod
    async def _update_data(self, data: dict, update_data: dict) -> dict: ...
    async def async_create_item(self, data: dict) -> dict: ...
    async def async_update_item(self, item_id: str, updates: dict) -> dict: ...
    async def async_delete_item(self, item_id: str) -> None: ...
    def _async_schedule_save(self) -> None: ...
    def _data_to_save(self) -> dict: ...

class IDLessCollection(ObservableCollection):
    counter: int
    async def async_load(self, data: list[dict]) -> None: ...

def sync_entity_lifecycle(hass: HomeAssistant, domain: str, platform: str, entity_component: EntityComponent, collection: ObservableCollection, create_entity: Callable[[dict], Entity]) -> None: ...

class StorageCollectionWebsocket:
    storage_collection: Any
    api_prefix: Any
    model_name: Any
    create_schema: Any
    update_schema: Any
    def __init__(self, storage_collection: StorageCollection, api_prefix: str, model_name: str, create_schema: dict, update_schema: dict) -> None: ...
    @property
    def item_id_key(self) -> str: ...
    def async_setup(self, hass: HomeAssistant, *, create_list: bool = ..., create_create: bool = ...) -> None: ...
    def ws_list_item(self, hass: HomeAssistant, connection: websocket_api.ActiveConnection, msg: dict) -> None: ...
    async def ws_create_item(self, hass: HomeAssistant, connection: websocket_api.ActiveConnection, msg: dict) -> None: ...
    async def ws_update_item(self, hass: HomeAssistant, connection: websocket_api.ActiveConnection, msg: dict) -> None: ...
    async def ws_delete_item(self, hass: HomeAssistant, connection: websocket_api.ActiveConnection, msg: dict) -> None: ...



from . import RitualsDataUpdateCoordinator as RitualsDataUpdateCoordinator
from .const import COORDINATORS as COORDINATORS, DEVICES as DEVICES, DOMAIN as DOMAIN
from .entity import DiffuserEntity as DiffuserEntity
from homeassistant.components.select import SelectEntity as SelectEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import AREA_SQUARE_METERS as AREA_SQUARE_METERS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from pyrituals import Diffuser as Diffuser
from typing import Any

ROOM_SIZE_SUFFIX: str

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class DiffuserRoomSize(DiffuserEntity, SelectEntity):
    _attr_icon: str
    _attr_unit_of_measurement: Any
    _attr_options: Any
    _attr_entity_registry_enabled_default: Any
    def __init__(self, diffuser: Diffuser, coordinator: RitualsDataUpdateCoordinator) -> None: ...
    @property
    def current_option(self) -> str: ...
    async def async_select_option(self, option: str) -> None: ...



from aiohttp.web import Application as Application
from aiohttp.web_urldispatcher import AbstractResource as AbstractResource
from homeassistant.const import HTTP_HEADER_X_REQUESTED_WITH as HTTP_HEADER_X_REQUESTED_WITH
from homeassistant.core import callback as callback
from typing import Any, Final

ALLOWED_CORS_HEADERS: Final[list[str]]
VALID_CORS_TYPES: Final[Any]

def setup_cors(app: Application, origins: list[str]) -> None: ...



from aiohttp.web import Application as Application, Request as Request, StreamResponse as StreamResponse
from collections.abc import Callable as Callable
from homeassistant.core import callback as callback
from typing import Any, Final

_LOGGER: Any
FILTERS: Final[Any]

def setup_security_filter(app: Application) -> None: ...



from . import DOMAIN as DOMAIN, SERVICE_CLOSE_COVER as SERVICE_CLOSE_COVER, SERVICE_OPEN_COVER as SERVICE_OPEN_COVER
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers import intent as intent

INTENT_OPEN_COVER: str
INTENT_CLOSE_COVER: str

async def async_setup_intents(hass: HomeAssistant) -> None: ...



import argparse
from homeassistant.const import REQUIRED_PYTHON_VER as REQUIRED_PYTHON_VER, RESTART_EXIT_CODE as RESTART_EXIT_CODE, __version__ as __version__

def validate_python() -> None: ...
def ensure_config_path(config_dir: str) -> None: ...
def get_arguments() -> argparse.Namespace: ...
def daemonize() -> None: ...
def check_pid(pid_file: str) -> None: ...
def write_pid(pid_file: str) -> None: ...
def closefds_osx(min_fd: int, max_fd: int) -> None: ...
def cmdline() -> list[str]: ...
def try_to_restart() -> None: ...
def main() -> int: ...



from . import PiHoleEntity as PiHoleEntity
from .const import DATA_KEY_API as DATA_KEY_API, DATA_KEY_COORDINATOR as DATA_KEY_COORDINATOR
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class PiHoleBinarySensor(PiHoleEntity, BinarySensorEntity):
    @property
    def name(self) -> str: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def is_on(self) -> bool: ...



from .const import AIOSHELLY_DEVICE_TIMEOUT_SEC as AIOSHELLY_DEVICE_TIMEOUT_SEC, DOMAIN as DOMAIN
from .utils import get_coap_context as get_coap_context, get_device_sleep_period as get_device_sleep_period
from homeassistant import config_entries as config_entries, core as core
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME, HTTP_UNAUTHORIZED as HTTP_UNAUTHORIZED
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.typing import DiscoveryInfoType as DiscoveryInfoType
from typing import Any, Final

_LOGGER: Final[Any]
HOST_SCHEMA: Final[Any]
HTTP_CONNECT_ERRORS: Final[Any]

async def validate_input(hass: core.HomeAssistant, host: str, data: dict[str, Any]) -> dict[str, Any]: ...

class ConfigFlow(config_entries.ConfigFlow):
    VERSION: int
    host: str
    info: dict[str, Any]
    device_info: dict[str, Any]
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_credentials(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_zeroconf(self, discovery_info: DiscoveryInfoType) -> FlowResult: ...
    async def async_step_confirm_discovery(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def _async_get_info(self, host: str) -> dict[str, Any]: ...



from .const import DOMAIN as DOMAIN
from .renault_coordinator import RenaultDataUpdateCoordinator as RenaultDataUpdateCoordinator
from datetime import timedelta
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from renault_api.kamereon import models as models
from renault_api.renault_vehicle import RenaultVehicle as RenaultVehicle
from typing import Any

LOGGER: Any

class RenaultVehicleProxy:
    hass: Any
    _vehicle: Any
    _details: Any
    _device_info: Any
    coordinators: Any
    hvac_target_temperature: int
    _scan_interval: Any
    def __init__(self, hass: HomeAssistant, vehicle: RenaultVehicle, details: models.KamereonVehicleDetails, scan_interval: timedelta) -> None: ...
    @property
    def details(self) -> models.KamereonVehicleDetails: ...
    @property
    def device_info(self) -> DeviceInfo: ...
    async def async_initialise(self) -> None: ...
    async def endpoint_available(self, endpoint: str) -> bool: ...
    async def get_battery_status(self) -> models.KamereonVehicleBatteryStatusData: ...
    async def get_charge_mode(self) -> models.KamereonVehicleChargeModeData: ...
    async def get_cockpit(self) -> models.KamereonVehicleCockpitData: ...
    async def get_hvac_status(self) -> models.KamereonVehicleHvacStatusData: ...



from .const import DOMAIN as DOMAIN, LOGGER as LOGGER, SCAN_INTERVAL as SCAN_INTERVAL, SERVICE_AIR_QUALITY as SERVICE_AIR_QUALITY, SERVICE_POLLEN as SERVICE_POLLEN
from ambee import AirQuality as AirQuality, Pollen as Pollen
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_API_KEY as CONF_API_KEY, CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import ConfigEntryAuthFailed as ConfigEntryAuthFailed
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any

PLATFORMS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...



from . import get_hyperion_device_id as get_hyperion_device_id, get_hyperion_unique_id as get_hyperion_unique_id, listen_for_instance_updates as listen_for_instance_updates
from .const import CONF_INSTANCE_CLIENTS as CONF_INSTANCE_CLIENTS, DOMAIN as DOMAIN, HYPERION_MANUFACTURER_NAME as HYPERION_MANUFACTURER_NAME, HYPERION_MODEL_NAME as HYPERION_MODEL_NAME, NAME_SUFFIX_HYPERION_CAMERA as NAME_SUFFIX_HYPERION_CAMERA, SIGNAL_ENTITY_REMOVE as SIGNAL_ENTITY_REMOVE, TYPE_HYPERION_CAMERA as TYPE_HYPERION_CAMERA
from aiohttp import web as web
from collections.abc import AsyncGenerator
from homeassistant.components.camera import Camera as Camera, DEFAULT_CONTENT_TYPE as DEFAULT_CONTENT_TYPE, async_get_still_stream as async_get_still_stream
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from hyperion import client as client
from typing import Any

_LOGGER: Any
IMAGE_STREAM_JPG_SENTINEL: str

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class HyperionCamera(Camera):
    _unique_id: Any
    _name: Any
    _device_id: Any
    _instance_name: Any
    _client: Any
    _image_cond: Any
    _image: Any
    _image_stream_clients: int
    _client_callbacks: Any
    def __init__(self, server_id: str, instance_num: int, instance_name: str, hyperion_client: client.HyperionClient) -> None: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def is_on(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    async def _update_imagestream(self, img: Union[dict[str, Any], None] = ...) -> None: ...
    async def _async_wait_for_camera_image(self) -> Union[bytes, None]: ...
    is_streaming: bool
    async def _start_image_streaming_for_client(self) -> bool: ...
    async def _stop_image_streaming_for_client(self) -> None: ...
    async def _image_streaming(self) -> AsyncGenerator: ...
    async def async_camera_image(self) -> Union[bytes, None]: ...
    async def handle_async_mjpeg_stream(self, request: web.Request) -> Union[web.StreamResponse, None]: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    @property
    def device_info(self) -> DeviceInfo: ...

CAMERA_TYPES: Any



import enum
from .dt import as_local as as_local, utcnow as utcnow
from collections.abc import Iterable, KeysView
from datetime import timedelta
from typing import Any, Callable, TypeVar

T = TypeVar('T')
U = TypeVar('U')
ENUM_T = TypeVar('ENUM_T', bound=enum.Enum)
RE_SANITIZE_FILENAME: Any
RE_SANITIZE_PATH: Any

def raise_if_invalid_filename(filename: str) -> None: ...
def raise_if_invalid_path(path: str) -> None: ...
def sanitize_filename(filename: str) -> str: ...
def sanitize_path(path: str) -> str: ...
def slugify(text: Union[str, None], *, separator: str = ...) -> str: ...
def repr_helper(inp: Any) -> str: ...
def convert(value: Union[T, None], to_type: Callable[[T], U], default: Union[U, None] = ...) -> Union[U, None]: ...
def ensure_unique_string(preferred_string: str, current_strings: Union[Iterable[str], KeysView[str]]) -> str: ...
def get_local_ip() -> str: ...
def get_random_string(length: int = ...) -> str: ...

class OrderedEnum(enum.Enum):
    def __ge__(self, other: ENUM_T) -> bool: ...
    def __gt__(self, other: ENUM_T) -> bool: ...
    def __le__(self, other: ENUM_T) -> bool: ...
    def __lt__(self, other: ENUM_T) -> bool: ...

class Throttle:
    min_time: Any
    limit_no_throttle: Any
    def __init__(self, min_time: timedelta, limit_no_throttle: Union[timedelta, None] = ...) -> None: ...
    def __call__(self, method: Callable) -> Callable: ...



from . import LcnEntity as LcnEntity
from .const import CONF_DOMAIN_DATA as CONF_DOMAIN_DATA, CONF_LOCKABLE as CONF_LOCKABLE, CONF_MAX_TEMP as CONF_MAX_TEMP, CONF_MIN_TEMP as CONF_MIN_TEMP, CONF_SETPOINT as CONF_SETPOINT
from .helpers import DeviceConnectionType as DeviceConnectionType, InputType as InputType, get_device_connection as get_device_connection
from homeassistant.components.climate import ClimateEntity as ClimateEntity, const as const
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_TEMPERATURE as ATTR_TEMPERATURE, CONF_ADDRESS as CONF_ADDRESS, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITIES as CONF_ENTITIES, CONF_SOURCE as CONF_SOURCE, CONF_UNIT_OF_MEASUREMENT as CONF_UNIT_OF_MEASUREMENT, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

PARALLEL_UPDATES: int

def create_lcn_climate_entity(hass: HomeAssistant, entity_config: ConfigType, config_entry: ConfigEntry) -> LcnEntity: ...
async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class LcnClimate(LcnEntity, ClimateEntity):
    variable: Any
    setpoint: Any
    unit: Any
    regulator_id: Any
    is_lockable: Any
    _max_temp: Any
    _min_temp: Any
    _current_temperature: Any
    _target_temperature: Any
    _is_on: bool
    def __init__(self, config: ConfigType, entry_id: str, device_connection: DeviceConnectionType) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def temperature_unit(self) -> str: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature(self) -> Union[float, None]: ...
    @property
    def hvac_mode(self) -> str: ...
    @property
    def hvac_modes(self) -> list[str]: ...
    @property
    def max_temp(self) -> float: ...
    @property
    def min_temp(self) -> float: ...
    async def async_set_hvac_mode(self, hvac_mode: str) -> None: ...
    async def async_set_temperature(self, **kwargs: Any) -> None: ...
    def input_received(self, input_obj: InputType) -> None: ...



from collections.abc import Hashable
from datetime import datetime, timedelta
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from typing import Any, Callable

_LOGGER: Any

class KeyedRateLimit:
    hass: Any
    _last_triggered: Any
    _rate_limit_timers: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    def async_has_timer(self, key: Hashable) -> bool: ...
    def async_triggered(self, key: Hashable, now: Union[datetime, None] = ...) -> None: ...
    def async_cancel_timer(self, key: Hashable) -> None: ...
    def async_remove(self) -> None: ...
    def async_schedule_action(self, key: Hashable, rate_limit: Union[timedelta, None], now: datetime, action: Callable, *args: Any) -> Union[datetime, None]: ...



from typing import Any

DOMAIN: str
LOGGER: Any
INTEGRATION_TYPE_GEOGRAPHY_COORDS: str
INTEGRATION_TYPE_GEOGRAPHY_NAME: str
INTEGRATION_TYPE_NODE_PRO: str
CONF_CITY: str
CONF_COUNTRY: str
CONF_GEOGRAPHIES: str
CONF_INTEGRATION_TYPE: str
DATA_COORDINATOR: str



from .const import ATTR_COMMAND_CLASS as ATTR_COMMAND_CLASS, ATTR_ENDPOINT as ATTR_ENDPOINT, ATTR_PROPERTY as ATTR_PROPERTY, ATTR_PROPERTY_KEY as ATTR_PROPERTY_KEY, CONF_DATA_COLLECTION_OPTED_IN as CONF_DATA_COLLECTION_OPTED_IN, DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers.device_registry import DeviceRegistry as DeviceRegistry
from homeassistant.helpers.entity_registry import EntityRegistry as EntityRegistry
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.model.node import Node as ZwaveNode
from zwave_js_server.model.value import Value as ZwaveValue

def get_value_of_zwave_value(value: Union[ZwaveValue, None]) -> Union[Any, None]: ...
async def async_enable_statistics(client: ZwaveClient) -> None: ...
def update_data_collection_preference(hass: HomeAssistant, entry: ConfigEntry, preference: bool) -> None: ...
def get_unique_id(home_id: str, value_id: str) -> str: ...
def get_device_id(client: ZwaveClient, node: ZwaveNode) -> tuple[str, str]: ...
def get_home_and_node_id_from_device_id(device_id: tuple[str, ...]) -> list[str]: ...
def async_get_node_from_device_id(hass: HomeAssistant, device_id: str, dev_reg: Union[DeviceRegistry, None] = ...) -> ZwaveNode: ...
def async_get_node_from_entity_id(hass: HomeAssistant, entity_id: str, ent_reg: Union[EntityRegistry, None] = ..., dev_reg: Union[DeviceRegistry, None] = ...) -> ZwaveNode: ...
def get_zwave_value_from_config(node: ZwaveNode, config: ConfigType) -> ZwaveValue: ...
def async_get_node_status_sensor_entity_id(hass: HomeAssistant, device_id: str, ent_reg: Union[EntityRegistry, None] = ..., dev_reg: Union[DeviceRegistry, None] = ...) -> str: ...



from . import ShellyDeviceWrapper as ShellyDeviceWrapper
from .const import AIOSHELLY_DEVICE_TIMEOUT_SEC as AIOSHELLY_DEVICE_TIMEOUT_SEC, COAP as COAP, DATA_CONFIG_ENTRY as DATA_CONFIG_ENTRY, DOMAIN as DOMAIN, KELVIN_MAX_VALUE as KELVIN_MAX_VALUE, KELVIN_MIN_VALUE_COLOR as KELVIN_MIN_VALUE_COLOR, KELVIN_MIN_VALUE_WHITE as KELVIN_MIN_VALUE_WHITE, SHBLB_1_RGB_EFFECTS as SHBLB_1_RGB_EFFECTS, STANDARD_RGB_EFFECTS as STANDARD_RGB_EFFECTS
from .entity import ShellyBlockEntity as ShellyBlockEntity
from .utils import async_remove_shelly_entity as async_remove_shelly_entity
from aioshelly import Block as Block
from homeassistant.components.light import ATTR_BRIGHTNESS as ATTR_BRIGHTNESS, ATTR_COLOR_TEMP as ATTR_COLOR_TEMP, ATTR_EFFECT as ATTR_EFFECT, ATTR_RGBW_COLOR as ATTR_RGBW_COLOR, ATTR_RGB_COLOR as ATTR_RGB_COLOR, COLOR_MODE_BRIGHTNESS as COLOR_MODE_BRIGHTNESS, COLOR_MODE_COLOR_TEMP as COLOR_MODE_COLOR_TEMP, COLOR_MODE_ONOFF as COLOR_MODE_ONOFF, COLOR_MODE_RGB as COLOR_MODE_RGB, COLOR_MODE_RGBW as COLOR_MODE_RGBW, LightEntity as LightEntity, SUPPORT_EFFECT as SUPPORT_EFFECT, brightness_supported as brightness_supported
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.color import color_temperature_kelvin_to_mired as color_temperature_kelvin_to_mired, color_temperature_mired_to_kelvin as color_temperature_mired_to_kelvin
from typing import Any, Final

_LOGGER: Final[Any]

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class ShellyLight(ShellyBlockEntity, LightEntity):
    control_result: Any
    mode_result: Any
    _supported_color_modes: Any
    _supported_features: int
    _min_kelvin: Any
    _max_kelvin: Any
    def __init__(self, wrapper: ShellyDeviceWrapper, block: Block) -> None: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def is_on(self) -> bool: ...
    @property
    def mode(self) -> str: ...
    @property
    def brightness(self) -> int: ...
    @property
    def color_mode(self) -> str: ...
    @property
    def rgb_color(self) -> tuple[int, int, int]: ...
    @property
    def rgbw_color(self) -> tuple[int, int, int, int]: ...
    @property
    def color_temp(self) -> int: ...
    @property
    def min_mireds(self) -> int: ...
    @property
    def max_mireds(self) -> int: ...
    @property
    def supported_color_modes(self) -> Union[set, None]: ...
    @property
    def effect_list(self) -> Union[list[str], None]: ...
    @property
    def effect(self) -> Union[str, None]: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def set_light_mode(self, set_mode: Union[str, None]) -> bool: ...
    def _update_callback(self) -> None: ...



from concurrent.futures import ThreadPoolExecutor
from homeassistant.util.thread import async_raise as async_raise
from threading import Thread
from typing import Any

_LOGGER: Any
MAX_LOG_ATTEMPTS: int
_JOIN_ATTEMPTS: int
EXECUTOR_SHUTDOWN_TIMEOUT: int

def _log_thread_running_at_shutdown(name: str, ident: int) -> None: ...
def join_or_interrupt_threads(threads: set[Thread], timeout: float, log: bool) -> set[Thread]: ...

class InterruptibleThreadPoolExecutor(ThreadPoolExecutor):
    _shutdown: bool
    def shutdown(self, *args, **kwargs) -> None: ...
    def join_threads_or_timeout(self) -> None: ...



from .const import SUBCAT_ALL as SUBCAT_ALL
from .models import PermissionLookup as PermissionLookup
from .types import CategoryType as CategoryType, SubCategoryDict as SubCategoryDict, ValueType as ValueType
from typing import Callable, Dict, Optional

LookupFunc = Callable[[PermissionLookup, SubCategoryDict, str], Optional[ValueType]]
SubCatLookupType = Dict[str, LookupFunc]

def lookup_all(perm_lookup: PermissionLookup, lookup_dict: SubCategoryDict, object_id: str) -> ValueType: ...
def compile_policy(policy: CategoryType, subcategories: SubCatLookupType, perm_lookup: PermissionLookup) -> Callable[[str, str], bool]: ...
def _gen_dict_test_func(perm_lookup: PermissionLookup, lookup_func: LookupFunc, lookup_dict: SubCategoryDict) -> Callable[[str, str], Union[bool, None]]: ...
def test_all(policy: CategoryType, key: str) -> bool: ...



import ruamel.yaml
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.util.yaml import secret_yaml as secret_yaml
from ruamel.yaml.constructor import SafeConstructor
from typing import Any, Union

_LOGGER: Any
JSON_TYPE = Union[list, dict, str]

class ExtSafeConstructor(SafeConstructor):
    name: Union[str, None]

class UnsupportedYamlError(HomeAssistantError): ...
class WriteError(HomeAssistantError): ...

def _include_yaml(constructor: ExtSafeConstructor, node: ruamel.yaml.nodes.Node) -> JSON_TYPE: ...
def _yaml_unsupported(constructor: ExtSafeConstructor, node: ruamel.yaml.nodes.Node) -> None: ...
def object_to_yaml(data: JSON_TYPE) -> str: ...
def yaml_to_object(data: str) -> JSON_TYPE: ...
def load_yaml(fname: str, round_trip: bool = ...) -> JSON_TYPE: ...
def save_yaml(fname: str, data: JSON_TYPE) -> None: ...



from homeassistant.const import UNIT_NOT_RECOGNIZED_TEMPLATE as UNIT_NOT_RECOGNIZED_TEMPLATE, VOLUME as VOLUME, VOLUME_FLUID_OUNCE as VOLUME_FLUID_OUNCE, VOLUME_GALLONS as VOLUME_GALLONS, VOLUME_LITERS as VOLUME_LITERS, VOLUME_MILLILITERS as VOLUME_MILLILITERS

VALID_UNITS: tuple[str, ...]

def __liter_to_gallon(liter: float) -> float: ...
def __gallon_to_liter(gallon: float) -> float: ...
def convert(volume: float, from_unit: str, to_unit: str) -> float: ...



from . import Recorder as Recorder
from .const import MAX_ROWS_TO_PURGE as MAX_ROWS_TO_PURGE
from .models import Events as Events, RecorderRuns as RecorderRuns, States as States
from .repack import repack_database as repack_database
from .util import retryable_database_job as retryable_database_job, session_scope as session_scope
from datetime import datetime
from sqlalchemy.orm.session import Session as Session
from typing import Any, Callable

_LOGGER: Any

def purge_old_data(instance: Recorder, purge_before: datetime, repack: bool, apply_filter: bool = ...) -> bool: ...
def _select_event_ids_to_purge(session: Session, purge_before: datetime) -> list[int]: ...
def _select_state_ids_to_purge(session: Session, purge_before: datetime, event_ids: list[int]) -> list[int]: ...
def _purge_state_ids(session: Session, state_ids: list[int]) -> None: ...
def _purge_event_ids(session: Session, event_ids: list[int]) -> None: ...
def _purge_old_recorder_runs(instance: Recorder, session: Session, purge_before: datetime) -> None: ...
def _purge_filtered_data(instance: Recorder, session: Session) -> bool: ...
def _purge_filtered_states(session: Session, excluded_entity_ids: list[str]) -> None: ...
def _purge_filtered_events(session: Session, excluded_event_types: list[str]) -> None: ...
def purge_entity_data(instance: Recorder, entity_filter: Callable[[str], bool]) -> bool: ...



import asyncio
from .const import ADDON_SLUG as ADDON_SLUG, CONF_ADDON_DEVICE as CONF_ADDON_DEVICE, CONF_ADDON_NETWORK_KEY as CONF_ADDON_NETWORK_KEY, DOMAIN as DOMAIN, LOGGER as LOGGER
from enum import Enum
from homeassistant.components.hassio import async_create_snapshot as async_create_snapshot, async_get_addon_discovery_info as async_get_addon_discovery_info, async_get_addon_info as async_get_addon_info, async_install_addon as async_install_addon, async_restart_addon as async_restart_addon, async_set_addon_options as async_set_addon_options, async_start_addon as async_start_addon, async_stop_addon as async_stop_addon, async_uninstall_addon as async_uninstall_addon, async_update_addon as async_update_addon
from homeassistant.components.hassio.handler import HassioAPIError as HassioAPIError
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers.singleton import singleton as singleton
from typing import Any, Callable, TypeVar

F = TypeVar('F', bound=Callable[..., Any])
DATA_ADDON_MANAGER: Any

def get_addon_manager(hass: HomeAssistant) -> AddonManager: ...
def api_error(error_message: str) -> Callable[[F], F]: ...

class AddonInfo:
    options: dict[str, Any]
    state: AddonState
    update_available: bool
    version: Union[str, None]

class AddonState(Enum):
    NOT_INSTALLED: str
    INSTALLING: str
    UPDATING: str
    NOT_RUNNING: str
    RUNNING: str

class AddonManager:
    _hass: Any
    _install_task: Any
    _restart_task: Any
    _start_task: Any
    _update_task: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    def task_in_progress(self) -> bool: ...
    async def async_get_addon_discovery_info(self) -> dict: ...
    async def async_get_addon_info(self) -> AddonInfo: ...
    def async_get_addon_state(self, addon_info: dict[str, Any]) -> AddonState: ...
    async def async_set_addon_options(self, config: dict) -> None: ...
    async def async_install_addon(self) -> None: ...
    def async_schedule_install_addon(self, catch_error: bool = ...) -> asyncio.Task: ...
    def async_schedule_install_setup_addon(self, usb_path: str, network_key: str, catch_error: bool = ...) -> asyncio.Task: ...
    async def async_uninstall_addon(self) -> None: ...
    async def async_update_addon(self) -> None: ...
    def async_schedule_update_addon(self, catch_error: bool = ...) -> asyncio.Task: ...
    async def async_start_addon(self) -> None: ...
    async def async_restart_addon(self) -> None: ...
    def async_schedule_start_addon(self, catch_error: bool = ...) -> asyncio.Task: ...
    def async_schedule_restart_addon(self, catch_error: bool = ...) -> asyncio.Task: ...
    async def async_stop_addon(self) -> None: ...
    async def async_configure_addon(self, usb_path: str, network_key: str) -> None: ...
    def async_schedule_setup_addon(self, usb_path: str, network_key: str, catch_error: bool = ...) -> asyncio.Task: ...
    async def async_create_snapshot(self) -> None: ...
    def _async_schedule_addon_operation(self, *funcs: Callable, catch_error: bool = ...) -> asyncio.Task: ...

class AddonError(HomeAssistantError): ...



DOMAIN: str
SIGN_ARIES: str
SIGN_TAURUS: str
SIGN_GEMINI: str
SIGN_CANCER: str
SIGN_LEO: str
SIGN_VIRGO: str
SIGN_LIBRA: str
SIGN_SCORPIO: str
SIGN_SAGITTARIUS: str
SIGN_CAPRICORN: str
SIGN_AQUARIUS: str
SIGN_PISCES: str
ELEMENT_FIRE: str
ELEMENT_AIR: str
ELEMENT_EARTH: str
ELEMENT_WATER: str
MODALITY_CARDINAL: str
MODALITY_FIXED: str
MODALITY_MUTABLE: str
ATTR_ELEMENT: str
ATTR_MODALITY: str



from .const import DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .entity import ZWaveBaseEntity as ZWaveBaseEntity
from .helpers import get_value_of_zwave_value as get_value_of_zwave_value
from homeassistant.components.climate import ClimateEntity as ClimateEntity, DEFAULT_MAX_TEMP as DEFAULT_MAX_TEMP, DEFAULT_MIN_TEMP as DEFAULT_MIN_TEMP
from homeassistant.components.climate.const import ATTR_HVAC_MODE as ATTR_HVAC_MODE, ATTR_TARGET_TEMP_HIGH as ATTR_TARGET_TEMP_HIGH, ATTR_TARGET_TEMP_LOW as ATTR_TARGET_TEMP_LOW, CURRENT_HVAC_COOL as CURRENT_HVAC_COOL, CURRENT_HVAC_FAN as CURRENT_HVAC_FAN, CURRENT_HVAC_HEAT as CURRENT_HVAC_HEAT, CURRENT_HVAC_IDLE as CURRENT_HVAC_IDLE, HVAC_MODE_COOL as HVAC_MODE_COOL, HVAC_MODE_DRY as HVAC_MODE_DRY, HVAC_MODE_FAN_ONLY as HVAC_MODE_FAN_ONLY, HVAC_MODE_HEAT as HVAC_MODE_HEAT, HVAC_MODE_HEAT_COOL as HVAC_MODE_HEAT_COOL, HVAC_MODE_OFF as HVAC_MODE_OFF, PRESET_NONE as PRESET_NONE, SUPPORT_FAN_MODE as SUPPORT_FAN_MODE, SUPPORT_PRESET_MODE as SUPPORT_PRESET_MODE, SUPPORT_TARGET_TEMPERATURE as SUPPORT_TARGET_TEMPERATURE, SUPPORT_TARGET_TEMPERATURE_RANGE as SUPPORT_TARGET_TEMPERATURE_RANGE
from homeassistant.components.zwave_js.discovery_data_template import DynamicCurrentTempClimateDataTemplate as DynamicCurrentTempClimateDataTemplate
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_TEMPERATURE as ATTR_TEMPERATURE, PRECISION_TENTHS as PRECISION_TENTHS, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.temperature import convert_temperature as convert_temperature
from typing import Any
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.const import ThermostatSetpointType
from zwave_js_server.model.value import Value as ZwaveValue

ZW_HVAC_MODE_MAP: dict[int, str]
HVAC_CURRENT_MAP: dict[int, str]
ATTR_FAN_STATE: str

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
    _hvac_modes: Any
    _hvac_presets: Any
    _unit_value: Any
    _current_mode: Any
    _setpoint_values: Any
    _operating_state: Any
    _current_temp: Any
    _current_humidity: Any
    _fan_mode: Any
    _fan_state: Any
    _supported_features: int
    def __init__(self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo) -> None: ...
    def _setpoint_value(self, setpoint_type: ThermostatSetpointType) -> ZwaveValue: ...
    def _set_modes_and_presets(self) -> None: ...
    @property
    def _current_mode_setpoint_enums(self) -> list[Union[ThermostatSetpointType, None]]: ...
    @property
    def temperature_unit(self) -> str: ...
    @property
    def precision(self) -> float: ...
    @property
    def hvac_mode(self) -> str: ...
    @property
    def hvac_modes(self) -> list[str]: ...
    @property
    def hvac_action(self) -> Union[str, None]: ...
    @property
    def current_humidity(self) -> Union[int, None]: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature_high(self) -> Union[float, None]: ...
    @property
    def target_temperature_low(self) -> Union[float, None]: ...
    @property
    def preset_mode(self) -> Union[str, None]: ...
    @property
    def preset_modes(self) -> Union[list[str], None]: ...
    @property
    def fan_mode(self) -> Union[str, None]: ...
    @property
    def fan_modes(self) -> Union[list[str], None]: ...
    @property
    def extra_state_attributes(self) -> Union[dict[str, str], None]: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def min_temp(self) -> float: ...
    @property
    def max_temp(self) -> float: ...
    async def async_set_fan_mode(self, fan_mode: str) -> None: ...
    async def async_set_temperature(self, **kwargs: Any) -> None: ...
    async def async_set_hvac_mode(self, hvac_mode: str) -> None: ...
    async def async_set_preset_mode(self, preset_mode: str) -> None: ...

class DynamicCurrentTempClimate(ZWaveClimate):
    data_template: Any
    def __init__(self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo) -> None: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...



from . import ATTR_BRIGHTNESS_PCT as ATTR_BRIGHTNESS_PCT, ATTR_BRIGHTNESS_STEP_PCT as ATTR_BRIGHTNESS_STEP_PCT, DOMAIN as DOMAIN, brightness_supported as brightness_supported, get_supported_color_modes as get_supported_color_modes
from homeassistant.components.device_automation import toggle_entity as toggle_entity
from homeassistant.components.light import ATTR_FLASH as ATTR_FLASH, FLASH_SHORT as FLASH_SHORT, SUPPORT_FLASH as SUPPORT_FLASH, VALID_BRIGHTNESS_PCT as VALID_BRIGHTNESS_PCT, VALID_FLASH as VALID_FLASH
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_TYPE as CONF_TYPE, SERVICE_TURN_ON as SERVICE_TURN_ON
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant, HomeAssistantError as HomeAssistantError
from homeassistant.helpers.entity import get_supported_features as get_supported_features
from homeassistant.helpers.typing import ConfigType as ConfigType, TemplateVarsType as TemplateVarsType
from typing import Any

TYPE_BRIGHTNESS_INCREASE: str
TYPE_BRIGHTNESS_DECREASE: str
TYPE_FLASH: str
ACTION_SCHEMA: Any

async def async_call_action_from_config(hass: HomeAssistant, config: ConfigType, variables: TemplateVarsType, context: Context) -> None: ...
async def async_get_actions(hass: HomeAssistant, device_id: str) -> list[dict]: ...
async def async_get_action_capabilities(hass: HomeAssistant, config: dict) -> dict: ...



from .objects import Input as Input
from typing import Any

class UndefinedSubstitution(Exception):
    input: Any
    def __init__(self, input_name: str) -> None: ...

def extract_inputs(obj: Any) -> set[str]: ...
def _extract_inputs(obj: Any, found: set[str]) -> None: ...
def substitute(obj: Any, substitutions: dict[str, Any]) -> Any: ...



import voluptuous as vol
from .const import DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant import config_entries as config_entries
from homeassistant.const import CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers import aiohttp_client as aiohttp_client
from typing import Any

class FluNearYouFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    @property
    def data_schema(self) -> vol.Schema: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



from . import ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_LANGUAGE as ATTR_LANGUAGE, ATTR_MESSAGE as ATTR_MESSAGE, DOMAIN as DOMAIN
from homeassistant.components.notify import BaseNotificationService as BaseNotificationService, PLATFORM_SCHEMA as PLATFORM_SCHEMA
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import split_entity_id as split_entity_id
from typing import Any

CONF_MEDIA_PLAYER: str
CONF_TTS_SERVICE: str
_LOGGER: Any

async def async_get_service(hass, config, discovery_info: Any | None = ...): ...

class TTSNotificationService(BaseNotificationService):
    _media_player: Any
    _language: Any
    def __init__(self, config) -> None: ...
    async def async_send_message(self, message: str = ..., **kwargs) -> None: ...



from homeassistant.helpers.device_registry import format_mac as format_mac
from huawei_lte_api.Connection import GetResponseType as GetResponseType

def get_device_macs(device_info: GetResponseType, wlan_settings: GetResponseType) -> list[str]: ...



from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.loader import Integration as Integration, MAX_LOAD_CONCURRENTLY as MAX_LOAD_CONCURRENTLY, async_get_config_flows as async_get_config_flows, async_get_integration as async_get_integration, bind_hass as bind_hass
from homeassistant.util.async_ import gather_with_concurrency as gather_with_concurrency
from homeassistant.util.json import load_json as load_json
from typing import Any

_LOGGER: Any
TRANSLATION_LOAD_LOCK: str
TRANSLATION_FLATTEN_CACHE: str
LOCALE_EN: str

def recursive_flatten(prefix: Any, data: dict) -> dict[str, Any]: ...
def component_translation_path(component: str, language: str, integration: Integration) -> Union[str, None]: ...
def load_translations_files(translation_files: dict[str, str]) -> dict[str, dict[str, Any]]: ...
def _merge_resources(translation_strings: dict[str, dict[str, Any]], components: set[str], category: str) -> dict[str, dict[str, Any]]: ...
def _build_resources(translation_strings: dict[str, dict[str, Any]], components: set[str], category: str) -> dict[str, dict[str, Any]]: ...
async def async_get_component_strings(hass: HomeAssistant, language: str, components: set[str]) -> dict[str, Any]: ...

class _TranslationCache:
    hass: Any
    loaded: Any
    cache: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    async def async_fetch(self, language: str, category: str, components: set) -> list[dict[str, dict[str, Any]]]: ...
    async def _async_load(self, language: str, components: set) -> None: ...
    def _build_category_cache(self, language: str, components: set, translation_strings: dict[str, dict[str, Any]]) -> None: ...

async def async_get_translations(hass: HomeAssistant, language: str, category: str, integration: Union[str, None] = ..., config_flow: Union[bool, None] = ...) -> dict[str, Any]: ...



import asyncio
from .const import LOGGER as LOGGER
from aioguardian import Client as Client
from collections.abc import Awaitable
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from typing import Any, Callable

DEFAULT_UPDATE_INTERVAL: Any

class GuardianDataUpdateCoordinator(DataUpdateCoordinator[dict]):
    _api_coro: Any
    _api_lock: Any
    _client: Any
    def __init__(self, hass: HomeAssistant, client: Client, api_name: str, api_coro: Callable[..., Awaitable], api_lock: asyncio.Lock, valve_controller_uid: str) -> None: ...
    async def _async_update_data(self) -> dict[str, Any]: ...



from . import const as const, messages as messages
from .http import WebSocketAdapter as WebSocketAdapter
from homeassistant.auth.models import RefreshToken as RefreshToken, User as User
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError, Unauthorized as Unauthorized
from typing import Any, Callable

class ActiveConnection:
    logger: Any
    hass: Any
    send_message: Any
    user: Any
    refresh_token_id: Any
    subscriptions: Any
    last_id: int
    def __init__(self, logger: WebSocketAdapter, hass: HomeAssistant, send_message: Callable[[Union[str, dict[str, Any]]], None], user: User, refresh_token: RefreshToken) -> None: ...
    def context(self, msg: dict[str, Any]) -> Context: ...
    def send_result(self, msg_id: int, result: Union[Any, None] = ...) -> None: ...
    async def send_big_result(self, msg_id: int, result: Any) -> None: ...
    def send_error(self, msg_id: int, code: str, message: str) -> None: ...
    def async_handle(self, msg: dict[str, Any]) -> None: ...
    def async_close(self) -> None: ...
    def async_handle_exception(self, msg: dict[str, Any], err: Exception) -> None: ...



from collections import deque
from collections.abc import Generator
from contextvars import ContextVar
from homeassistant.helpers.typing import TemplateVarsType as TemplateVarsType
from typing import Any, Callable

class TraceElement:
    _child_key: Any
    _child_run_id: Any
    _error: Any
    path: Any
    _result: Any
    reuse_by_child: bool
    _timestamp: Any
    _variables: Any
    def __init__(self, variables: TemplateVarsType, path: str) -> None: ...
    def __repr__(self) -> str: ...
    def set_child_id(self, child_key: tuple[str, str], child_run_id: str) -> None: ...
    def set_error(self, ex: Exception) -> None: ...
    def set_result(self, **kwargs: Any) -> None: ...
    def update_result(self, **kwargs: Any) -> None: ...
    def as_dict(self) -> dict[str, Any]: ...

trace_cv: ContextVar[Union[dict[str, deque[TraceElement]], None]]
trace_stack_cv: ContextVar[Union[list[TraceElement], None]]
trace_path_stack_cv: ContextVar[Union[list[str], None]]
variables_cv: ContextVar[Union[Any, None]]
trace_id_cv: ContextVar[Union[tuple[tuple[str, str], str], None]]
script_execution_cv: ContextVar[Union[StopReason, None]]

def trace_id_set(trace_id: tuple[tuple[str, str], str]) -> None: ...
def trace_id_get() -> Union[tuple[tuple[str, str], str], None]: ...
def trace_stack_push(trace_stack_var: ContextVar, node: Any) -> None: ...
def trace_stack_pop(trace_stack_var: ContextVar) -> None: ...
def trace_stack_top(trace_stack_var: ContextVar) -> Union[Any, None]: ...
def trace_path_push(suffix: Union[str, list[str]]) -> int: ...
def trace_path_pop(count: int) -> None: ...
def trace_path_get() -> str: ...
def trace_append_element(trace_element: TraceElement, maxlen: Union[int, None] = ...) -> None: ...
def trace_get(clear: bool = ...) -> Union[dict[str, deque[TraceElement]], None]: ...
def trace_clear() -> None: ...
def trace_set_child_id(child_key: tuple[str, str], child_run_id: str) -> None: ...
def trace_set_result(**kwargs: Any) -> None: ...
def trace_update_result(**kwargs: Any) -> None: ...

class StopReason:
    script_execution: Union[str, None]

def script_execution_set(reason: str) -> None: ...
def script_execution_get() -> Union[str, None]: ...
def trace_path(suffix: Union[str, list[str]]) -> Generator: ...
def async_trace_path(suffix: Union[str, list[str]]) -> Callable: ...



from typing import Any

DOMAIN: str
ATTR_ENDPOINTS: str
ATTR_STREAMS: str
HLS_PROVIDER: str
RECORDER_PROVIDER: str
OUTPUT_FORMATS: Any
SEGMENT_CONTAINER_FORMAT: str
RECORDER_CONTAINER_FORMAT: str
AUDIO_CODECS: Any
FORMAT_CONTENT_TYPE: Any
OUTPUT_IDLE_TIMEOUT: int
NUM_PLAYLIST_SEGMENTS: int
MAX_SEGMENTS: int
TARGET_SEGMENT_DURATION: float
TARGET_PART_DURATION: float
SEGMENT_DURATION_ADJUSTER: float
MIN_SEGMENT_DURATION: Any
EXT_X_START: float
PACKETS_TO_WAIT_FOR_AUDIO: int
MAX_TIMESTAMP_GAP: int
MAX_MISSING_DTS: int
SOURCE_TIMEOUT: int
STREAM_RESTART_INCREMENT: int
STREAM_RESTART_RESET_TIME: int



from .const import DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .entity import ZWaveBaseEntity as ZWaveBaseEntity
from homeassistant.components.light import ATTR_BRIGHTNESS as ATTR_BRIGHTNESS, ATTR_COLOR_TEMP as ATTR_COLOR_TEMP, ATTR_HS_COLOR as ATTR_HS_COLOR, ATTR_RGBW_COLOR as ATTR_RGBW_COLOR, ATTR_TRANSITION as ATTR_TRANSITION, COLOR_MODE_BRIGHTNESS as COLOR_MODE_BRIGHTNESS, COLOR_MODE_COLOR_TEMP as COLOR_MODE_COLOR_TEMP, COLOR_MODE_HS as COLOR_MODE_HS, COLOR_MODE_RGBW as COLOR_MODE_RGBW, LightEntity as LightEntity, SUPPORT_TRANSITION as SUPPORT_TRANSITION
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.const import ColorComponent

LOGGER: Any
MULTI_COLOR_MAP: Any
TRANSITION_DURATION: str

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...
def byte_to_zwave_brightness(value: int) -> int: ...

class ZwaveLight(ZWaveBaseEntity, LightEntity):
    _supports_color: bool
    _supports_rgbw: bool
    _supports_color_temp: bool
    _hs_color: Any
    _rgbw_color: Any
    _color_mode: Any
    _color_temp: Any
    _min_mireds: int
    _max_mireds: int
    _warm_white: Any
    _cold_white: Any
    _supported_color_modes: Any
    _target_brightness: Any
    _target_color: Any
    _attr_supported_features: int
    supports_brightness_transition: Any
    supports_color_transition: Any
    def __init__(self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo) -> None: ...
    def on_value_update(self) -> None: ...
    @property
    def brightness(self) -> int: ...
    @property
    def color_mode(self) -> Union[str, None]: ...
    @property
    def is_on(self) -> bool: ...
    @property
    def hs_color(self) -> Union[tuple[float, float], None]: ...
    @property
    def rgbw_color(self) -> Union[tuple[int, int, int, int], None]: ...
    @property
    def color_temp(self) -> Union[int, None]: ...
    @property
    def min_mireds(self) -> int: ...
    @property
    def max_mireds(self) -> int: ...
    @property
    def supported_color_modes(self) -> Union[set, None]: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def _async_set_colors(self, colors: dict[ColorComponent, int], transition: Union[float, None] = ...) -> None: ...
    async def _async_set_color(self, color: ColorComponent, new_value: int, transition: Union[dict[str, str], None] = ...) -> None: ...
    async def _async_set_brightness(self, brightness: Union[int, None], transition: Union[float, None] = ...) -> None: ...
    def _calculate_color_values(self) -> None: ...
